      MODULE s_NQUADR

      CONTAINS

C*
      SUBROUTINE NQUADR(IDIM,Q,deriv,TA,TB,VALINI,VALUE,T0,H,COEF,
     1                  IRET,satpos, stdorb, act_sat)

CC
CC NAME       :  NQUADR
CC
CC PURPOSE    :  SOLVE NINT INTEGRALS USING A GAUSSIAN QUADRATURE
CC               ALGORITHM. RETURN THE DEFINITE INTEGRAL PLUS THE
CC               COEFFICIENTS OF THE SOLUTION
CC
CC PARAMETERS :
CC        IN  : IDIM    : NUMBER OF INTEGRALS TO BE PERFORMED       I*4
CC              Q       : ORDER OF METHOD                           I*4
CC              TA, TB  : INTERVAL BOUNDARIS                        R*8
CC              VALINI  : INITIAL VALUES AT LEFT INTERVAL BOUNDARY  R*8
CC              act_sat : active satellite                          I*4
CC        OUT : VALUE   : RESULT (DEFINITE INTEGRALS FROM TA TO TB) R*8
CC              T0      : ORIGIN OF DEVELOPMENT                     R*8
CC              H       : SCALING CONSTANT                          R*8
CC              COEF    : COEFFICIENTS                              R*8
CC              IRET    : RETURN CODE                               I*4
CC                        =0: EVERYTHING OK
CC                        =1: ORDER Q OUT OF RANGE
CC
CC REMARKS    :
CC
CC AUTHOR     :  G.BEUTLER
CC
CC VERSION    :  4.1
CC
CC CREATED    :  08-JUN-00
CC
CC CHANGES    :  08-FEB-01 : DI: RHS MADE ALLOCATABLE, USE M_BERN
CC               18-SEP-12 : RD: IMPLCIT NONE
CC               18-SEP-12 : RD: USE M_BERN,D_CONST WITH ONLY
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1998     UNIVERSITY OF BERN
CC               SWITZERLAND
CC
C*
      USE m_bern,  ONLY: i4b, r8b
!
      USE s_alcerr
      USE s_dminv
      USE s_polevn
      USE s_stdorbit_t

C      IMPLICIT REAL*8 (A-H,O-Z)
C      IMPLICIT INTEGER*4 (I-N)
      IMPLICIT NONE

      INTEGER(i4b), PARAMETER ::  MAXORD = 40

      INTEGER(i4b) :: IDIM,IRET,IAC,IQ,I,K,IK,KDIM,I0,K0

      REAL(r8b) :: TA,TB,T0,H,DET,TEXT,DT

      TYPE(t_satPos), DIMENSION(:), INTENT(INOUT), OPTIONAL :: satPos
      TYPE(t_stdOrb), DIMENSION(:),                OPTIONAL :: stdOrb
      INTEGER(i4b),                                OPTIONAL :: act_sat

      INTEGER*4 Q,QOLD,LHLP(MAXORD),MHLP(MAXORD)
      REAL*8    VALINI(*),VALUE(*),COEF(IDIM,*)
      REAL*8    MAT(MAXORD*MAXORD),TAU(MAXORD),WGT(MAXORD)
      REAL(r8b),DIMENSION(:),ALLOCATABLE,SAVE :: RHS
      INTEGER(i4b),SAVE :: IDIMOLD=-1

      INTERFACE
      SUBROUTINE deriv(tmjd, rhs, satpos, stdorb, act_sat)
        USE m_bern
        USE s_stdorbit_t
        REAL(r8b)                                             :: tmjd
        REAL(r8b)                                             :: rhs(*)
        TYPE(t_stdOrb), DIMENSION(:),                OPTIONAL :: stdOrb
        TYPE(t_satPos), DIMENSION(:), INTENT(INOUT), OPTIONAL :: satPos
        INTEGER(i4b),                                OPTIONAL :: act_sat
      END SUBROUTINE
      END INTERFACE

      DATA      QOLD/-1/

!
! INITIALIZE RHS
! --------------
      IF(IDIM/=IDIMOLD.OR.Q/=QOLD) THEN
        IF (ALLOCATED(RHS)) DEALLOCATE(RHS)
        ALLOCATE(RHS(IDIM*Q),STAT=IAC)
        CALL ALCERR(IAC,'RHS',(/IDIM*Q/),'NQUADR')
        IDIMOLD=IDIM
      ENDIF
C
C SET VERTICES, WEIGHTS
C ---------------------
!!!      WRITE(*,*)'Q,QOLD********************************=',Q,QOLD
      IF(Q.NE.QOLD)THEN
        IF(Q.EQ.1)THEN
          TAU( 1)=   0.0000000000000000D+00
          WGT( 1)=   0.2000000000000000D+01
        ELSE IF(Q.EQ.2)THEN
          TAU( 2)=   0.5773502691896257D+00
          WGT( 2)=   0.9999999999999998D+00
        ELSE IF(Q.EQ.3)THEN
          TAU( 2)=   0.0000000000000000D+00
          WGT( 2)=   0.8888888888888888D+00
          TAU( 3)=   0.7745966692414834D+00
          WGT( 3)=   0.5555555555555554D+00
        ELSE IF(Q.EQ.4)THEN
          TAU( 3)=   0.3399810435848563D+00
          WGT( 3)=   0.6521451548625462D+00
          TAU( 4)=   0.8611363115940526D+00
          WGT( 4)=   0.3478548451374539D+00
        ELSE IF(Q.EQ.5)THEN
          TAU( 3)=   0.0000000000000000D+00
          WGT( 3)=   0.5688888888888889D+00
          TAU( 4)=   0.5384693101056831D+00
          WGT( 4)=   0.4786286704993665D+00
          TAU( 5)=   0.9061798459386640D+00
          WGT( 5)=   0.2369268850561891D+00
        ELSE IF(Q.EQ.6)THEN
          TAU( 4)=   0.2386191860831969D+00
          WGT( 4)=   0.4679139345726910D+00
          TAU( 5)=   0.6612093864662645D+00
          WGT( 5)=   0.3607615730481387D+00
          TAU( 6)=   0.9324695142031521D+00
          WGT( 6)=   0.1713244923791703D+00
        ELSE IF(Q.EQ.7)THEN
          TAU( 4)=   0.0000000000000000D+00
          WGT( 4)=   0.4179591836734694D+00
          TAU( 5)=   0.4058451513773972D+00
          WGT( 5)=   0.3818300505051189D+00
          TAU( 6)=   0.7415311855993945D+00
          WGT( 6)=   0.2797053914892768D+00
          TAU( 7)=   0.9491079123427586D+00
          WGT( 7)=   0.1294849661688700D+00
        ELSE IF(Q.EQ.8)THEN
          TAU( 5)=   0.1834346424956498D+00
          WGT( 5)=   0.3626837833783619D+00
          TAU( 6)=   0.5255324099163290D+00
          WGT( 6)=   0.3137066458778873D+00
          TAU( 7)=   0.7966664774136267D+00
          WGT( 7)=   0.2223810344533744D+00
          TAU( 8)=   0.9602898564975362D+00
          WGT( 8)=   0.1012285362903758D+00
        ELSE IF(Q.EQ.9)THEN
          TAU( 5)=   0.0000000000000000D+00
          WGT( 5)=   0.3302393550012598D+00
          TAU( 6)=   0.3242534234038089D+00
          WGT( 6)=   0.3123470770400028D+00
          TAU( 7)=   0.6133714327005904D+00
          WGT( 7)=   0.2606106964029355D+00
          TAU( 8)=   0.8360311073266358D+00
          WGT( 8)=   0.1806481606948573D+00
          TAU( 9)=   0.9681602395076261D+00
          WGT( 9)=   0.8127438836157438D-01
        ELSE IF(Q.EQ.10)THEN
          TAU( 6)=   0.1488743389816312D+00
          WGT( 6)=   0.2955242247147529D+00
          TAU( 7)=   0.4333953941292472D+00
          WGT( 7)=   0.2692667193099962D+00
          TAU( 8)=   0.6794095682990244D+00
          WGT( 8)=   0.2190863625159821D+00
          TAU( 9)=   0.8650633666889845D+00
          WGT( 9)=   0.1494513491505805D+00
          TAU(10)=   0.9739065285171717D+00
          WGT(10)=   0.6667134430868805D-01
        ELSE IF(Q.EQ.11)THEN
          TAU( 6)=   0.0000000000000000D+00
          WGT( 6)=   0.2729250867779006D+00
          TAU( 7)=   0.2695431559523450D+00
          WGT( 7)=   0.2628045445102466D+00
          TAU( 8)=   0.5190961292068118D+00
          WGT( 8)=   0.2331937645919906D+00
          TAU( 9)=   0.7301520055740494D+00
          WGT( 9)=   0.1862902109277341D+00
          TAU(10)=   0.8870625997680953D+00
          WGT(10)=   0.1255803694649046D+00
          TAU(11)=   0.9782286581460570D+00
          WGT(11)=   0.5566856711617367D-01
        ELSE IF(Q.EQ.12)THEN
          TAU( 7)=   0.1252334085114689D+00
          WGT( 7)=   0.2491470458134027D+00
          TAU( 8)=   0.3678314989981802D+00
          WGT( 8)=   0.2334925365383549D+00
          TAU( 9)=   0.5873179542866175D+00
          WGT( 9)=   0.2031674267230659D+00
          TAU(10)=   0.7699026741943047D+00
          WGT(10)=   0.1600783285433462D+00
          TAU(11)=   0.9041172563704749D+00
          WGT(11)=   0.1069393259953186D+00
          TAU(12)=   0.9815606342467192D+00
          WGT(12)=   0.4717533638651181D-01
        ELSE IF(Q.EQ.13)THEN
          TAU( 7)=   0.0000000000000000D+00
          WGT( 7)=   0.2325515532308739D+00
          TAU( 8)=   0.2304583159551348D+00
          WGT( 8)=   0.2262831802628973D+00
          TAU( 9)=   0.4484927510364469D+00
          WGT( 9)=   0.2078160475368884D+00
          TAU(10)=   0.6423493394403402D+00
          WGT(10)=   0.1781459807619457D+00
          TAU(11)=   0.8015780907333099D+00
          WGT(11)=   0.1388735102197873D+00
          TAU(12)=   0.9175983992229780D+00
          WGT(12)=   0.9212149983772862D-01
          TAU(13)=   0.9841830547185881D+00
          WGT(13)=   0.4048400476531598D-01
        ELSE IF(Q.EQ.14)THEN
          TAU( 8)=   0.1080549487073437D+00
          WGT( 8)=   0.2152638534631578D+00
          TAU( 9)=   0.3191123689278897D+00
          WGT( 9)=   0.2051984637212956D+00
          TAU(10)=   0.5152486363581541D+00
          WGT(10)=   0.1855383974779378D+00
          TAU(11)=   0.6872929048116855D+00
          WGT(11)=   0.1572031671581936D+00
          TAU(12)=   0.8272013150697650D+00
          WGT(12)=   0.1215185706879031D+00
          TAU(13)=   0.9284348836635735D+00
          WGT(13)=   0.8015808715976018D-01
          TAU(14)=   0.9862838086968123D+00
          WGT(14)=   0.3511946033175189D-01
        ELSE IF(Q.EQ.15)THEN
          TAU( 8)=   0.0000000000000000D+00
          WGT( 8)=   0.2025782419255613D+00
          TAU( 9)=   0.2011940939974345D+00
          WGT( 9)=   0.1984314853271116D+00
          TAU(10)=   0.3941513470775634D+00
          WGT(10)=   0.1861610000155623D+00
          TAU(11)=   0.5709721726085388D+00
          WGT(11)=   0.1662692058169940D+00
          TAU(12)=   0.7244177313601701D+00
          WGT(12)=   0.1395706779261543D+00
          TAU(13)=   0.8482065834104272D+00
          WGT(13)=   0.1071592204671719D+00
          TAU(14)=   0.9372733924007058D+00
          WGT(14)=   0.7036604748810794D-01
          TAU(15)=   0.9879925180204854D+00
          WGT(15)=   0.3075324199611678D-01
        ELSE IF(Q.EQ.16)THEN
          TAU( 9)=   0.9501250983763744D-01
          WGT( 9)=   0.1894506104550685D+00
          TAU(10)=   0.2816035507792589D+00
          WGT(10)=   0.1826034150449236D+00
          TAU(11)=   0.4580167776572274D+00
          WGT(11)=   0.1691565193950026D+00
          TAU(12)=   0.6178762444026438D+00
          WGT(12)=   0.1495959888165768D+00
          TAU(13)=   0.7554044083550030D+00
          WGT(13)=   0.1246289712555339D+00
          TAU(14)=   0.8656312023878318D+00
          WGT(14)=   0.9515851168249283D-01
          TAU(15)=   0.9445750230732326D+00
          WGT(15)=   0.6225352393864783D-01
          TAU(16)=   0.9894009349916499D+00
          WGT(16)=   0.2715245941175409D-01
        ELSE IF(Q.EQ.17)THEN
          TAU( 9)=   0.0000000000000000D+00
          WGT( 9)=   0.1794464703562065D+00
          TAU(10)=   0.1784841814958479D+00
          WGT(10)=   0.1765627053669926D+00
          TAU(11)=   0.3512317634538763D+00
          WGT(11)=   0.1680041021564500D+00
          TAU(12)=   0.5126905370864769D+00
          WGT(12)=   0.1540457610768103D+00
          TAU(13)=   0.6576711592166907D+00
          WGT(13)=   0.1351363684685255D+00
          TAU(14)=   0.7815140038968014D+00
          WGT(14)=   0.1118838471934040D+00
          TAU(15)=   0.8802391537269859D+00
          WGT(15)=   0.8503614831717912D-01
          TAU(16)=   0.9506755217687678D+00
          WGT(16)=   0.5545952937398717D-01
          TAU(17)=   0.9905754753144174D+00
          WGT(17)=   0.2414830286854785D-01
        ELSE IF(Q.EQ.18)THEN
          TAU(10)=   0.8477501304173531D-01
          WGT(10)=   0.1691423829631435D+00
          TAU(11)=   0.2518862256915055D+00
          WGT(11)=   0.1642764837458328D+00
          TAU(12)=   0.4117511614628426D+00
          WGT(12)=   0.1546846751262652D+00
          TAU(13)=   0.5597708310739475D+00
          WGT(13)=   0.1406429146706506D+00
          TAU(14)=   0.6916870430603532D+00
          WGT(14)=   0.1225552067114785D+00
          TAU(15)=   0.8037049589725231D+00
          WGT(15)=   0.1009420441062872D+00
          TAU(16)=   0.8926024664975557D+00
          WGT(16)=   0.7642573025488912D-01
          TAU(17)=   0.9558239495713977D+00
          WGT(17)=   0.4971454889496989D-01
          TAU(18)=   0.9915651684209309D+00
          WGT(18)=   0.2161601352648342D-01
        ELSE IF(Q.EQ.19)THEN
          TAU(10)=   0.0000000000000000D+00
          WGT(10)=   0.1610544498487837D+00
          TAU(11)=   0.1603586456402254D+00
          WGT(11)=   0.1589688433939543D+00
          TAU(12)=   0.3165640999636298D+00
          WGT(12)=   0.1527660420658597D+00
          TAU(13)=   0.4645707413759609D+00
          WGT(13)=   0.1426067021736067D+00
          TAU(14)=   0.6005453046616810D+00
          WGT(14)=   0.1287539625393363D+00
          TAU(15)=   0.7209661773352294D+00
          WGT(15)=   0.1115666455473340D+00
          TAU(16)=   0.8227146565371428D+00
          WGT(16)=   0.9149002162245001D-01
          TAU(17)=   0.9031559036148179D+00
          WGT(17)=   0.6904454273764116D-01
          TAU(18)=   0.9602081521348300D+00
          WGT(18)=   0.4481422676569968D-01
          TAU(19)=   0.9924068438435845D+00
          WGT(19)=   0.1946178822972689D-01
        ELSE IF(Q.EQ.20)THEN
          TAU(11)=   0.7652652113349732D-01
          WGT(11)=   0.1527533871307259D+00
          TAU(12)=   0.2277858511416451D+00
          WGT(12)=   0.1491729864726038D+00
          TAU(13)=   0.3737060887154195D+00
          WGT(13)=   0.1420961093183822D+00
          TAU(14)=   0.5108670019508271D+00
          WGT(14)=   0.1316886384491765D+00
          TAU(15)=   0.6360536807265150D+00
          WGT(15)=   0.1181945319615185D+00
          TAU(16)=   0.7463319064601508D+00
          WGT(16)=   0.1019301198172405D+00
          TAU(17)=   0.8391169718222188D+00
          WGT(17)=   0.8327674157670481D-01
          TAU(18)=   0.9122344282513259D+00
          WGT(18)=   0.6267204833410908D-01
          TAU(19)=   0.9639719272779138D+00
          WGT(19)=   0.4060142980038697D-01
          TAU(20)=   0.9931285991850949D+00
          WGT(20)=   0.1761400713915221D-01
        ELSE IF(Q.EQ.21)THEN
          TAU(11)=   0.0000000000000000D+00
          WGT(11)=   0.1460811336496904D+00
          TAU(12)=   0.1455618541608951D+00
          WGT(12)=   0.1445244039899701D+00
          TAU(13)=   0.2880213168024011D+00
          WGT(13)=   0.1398873947910732D+00
          TAU(14)=   0.4243421202074388D+00
          WGT(14)=   0.1322689386333374D+00
          TAU(15)=   0.5516188358872198D+00
          WGT(15)=   0.1218314160537285D+00
          TAU(16)=   0.6671388041974123D+00
          WGT(16)=   0.1087972991671484D+00
          TAU(17)=   0.7684399634756779D+00
          WGT(17)=   0.9344442345603385D-01
          TAU(18)=   0.8533633645833173D+00
          WGT(18)=   0.7610011362837933D-01
          TAU(19)=   0.9200993341504008D+00
          WGT(19)=   0.5713442542685719D-01
          TAU(20)=   0.9672268385663063D+00
          WGT(20)=   0.3695378977085245D-01
          TAU(21)=   0.9937521706203896D+00
          WGT(21)=   0.1601722825777473D-01
        ELSE IF(Q.EQ.22)THEN
          TAU(12)=   0.6973927331972223D-01
          WGT(12)=   0.1392518728556320D+00
          TAU(13)=   0.2078604266882213D+00
          WGT(13)=   0.1365414983460151D+00
          TAU(14)=   0.3419358208920842D+00
          WGT(14)=   0.1311735047870623D+00
          TAU(15)=   0.4693558379867570D+00
          WGT(15)=   0.1232523768105123D+00
          TAU(16)=   0.5876404035069116D+00
          WGT(16)=   0.1129322960805392D+00
          TAU(17)=   0.6944872631866827D+00
          WGT(17)=   0.1004141444428810D+00
          TAU(18)=   0.7878168059792081D+00
          WGT(18)=   0.8594160621706773D-01
          TAU(19)=   0.8658125777203001D+00
          WGT(19)=   0.6979646842452046D-01
          TAU(20)=   0.9269567721871740D+00
          WGT(20)=   0.5229333515268328D-01
          TAU(21)=   0.9700604978354287D+00
          WGT(21)=   0.3377490158481419D-01
          TAU(22)=   0.9942945854823992D+00
          WGT(22)=   0.1462799529827234D-01
        ELSE IF(Q.EQ.23)THEN
          TAU(12)=   0.0000000000000000D+00
          WGT(12)=   0.1336545721861062D+00
          TAU(13)=   0.1332568242984661D+00
          WGT(13)=   0.1324620394046966D+00
          TAU(14)=   0.2641356809703450D+00
          WGT(14)=   0.1289057221880822D+00
          TAU(15)=   0.3903010380302908D+00
          WGT(15)=   0.1230490843067295D+00
          TAU(16)=   0.5095014778460075D+00
          WGT(16)=   0.1149966402224114D+00
          TAU(17)=   0.6196098757636461D+00
          WGT(17)=   0.1048920914645414D+00
          TAU(18)=   0.7186613631319502D+00
          WGT(18)=   0.9291576606003510D-01
          TAU(19)=   0.8048884016188399D+00
          WGT(19)=   0.7928141177671895D-01
          TAU(20)=   0.8767523582704416D+00
          WGT(20)=   0.6423242140852592D-01
          TAU(21)=   0.9329710868260160D+00
          WGT(21)=   0.4803767173108453D-01
          TAU(22)=   0.9725424712181152D+00
          WGT(22)=   0.3098800585697949D-01
          TAU(23)=   0.9947693349975522D+00
          WGT(23)=   0.1341185948714170D-01
        ELSE IF(Q.EQ.24)THEN
          TAU(13)=   0.6405689286260563D-01
          WGT(13)=   0.1279381953467522D+00
          TAU(14)=   0.1911188674736163D+00
          WGT(14)=   0.1258374563468283D+00
          TAU(15)=   0.3150426796961634D+00
          WGT(15)=   0.1216704729278034D+00
          TAU(16)=   0.4337935076260451D+00
          WGT(16)=   0.1155056680537255D+00
          TAU(17)=   0.5454214713888396D+00
          WGT(17)=   0.1074442701159657D+00
          TAU(18)=   0.6480936519369755D+00
          WGT(18)=   0.9761865210411391D-01
          TAU(19)=   0.7401241915785544D+00
          WGT(19)=   0.8619016153195325D-01
          TAU(20)=   0.8200019859739029D+00
          WGT(20)=   0.7334648141108027D-01
          TAU(21)=   0.8864155270044011D+00
          WGT(21)=   0.5929858491543680D-01
          TAU(22)=   0.9382745520027328D+00
          WGT(22)=   0.4427743881741976D-01
          TAU(23)=   0.9747285559713095D+00
          WGT(23)=   0.2853138862893372D-01
          TAU(24)=   0.9951872199970213D+00
          WGT(24)=   0.1234122979998731D-01
        ELSE IF(Q.EQ.25)THEN
          TAU(13)=   0.0000000000000000D+00
          WGT(13)=   0.1231760537267154D+00
          TAU(14)=   0.1228646926107104D+00
          WGT(14)=   0.1222424429903101D+00
          TAU(15)=   0.2438668837209884D+00
          WGT(15)=   0.1194557635357848D+00
          TAU(16)=   0.3611723058093879D+00
          WGT(16)=   0.1148582591457117D+00
          TAU(17)=   0.4730027314457150D+00
          WGT(17)=   0.1085196244742637D+00
          TAU(18)=   0.5776629302412229D+00
          WGT(18)=   0.1005359490670506D+00
          TAU(19)=   0.6735663684734684D+00
          WGT(19)=   0.9102826198296367D-01
          TAU(20)=   0.7592592630373576D+00
          WGT(20)=   0.8014070033500099D-01
          TAU(21)=   0.8334426287608340D+00
          WGT(21)=   0.6803833381235691D-01
          TAU(22)=   0.8949919978782753D+00
          WGT(22)=   0.5490469597583525D-01
          TAU(23)=   0.9429745712289743D+00
          WGT(23)=   0.4093915670130633D-01
          TAU(24)=   0.9766639214595175D+00
          WGT(24)=   0.2635498661503212D-01
          TAU(25)=   0.9955569697904981D+00
          WGT(25)=   0.1139379850102620D-01
        ELSE IF(Q.EQ.26)THEN
          TAU(14)=   0.5923009342931321D-01
          WGT(14)=   0.1183214152792623D+00
          TAU(15)=   0.1768588203568902D+00
          WGT(15)=   0.1166604434852966D+00
          TAU(16)=   0.2920048394859569D+00
          WGT(16)=   0.1133618165463196D+00
          TAU(17)=   0.4030517551234863D+00
          WGT(17)=   0.1084718405285766D+00
          TAU(18)=   0.5084407148245057D+00
          WGT(18)=   0.1020591610944254D+00
          TAU(19)=   0.6066922930176181D+00
          WGT(19)=   0.9421380035591413D-01
          TAU(20)=   0.6964272604199573D+00
          WGT(20)=   0.8504589431348521D-01
          TAU(21)=   0.7763859488206789D+00
          WGT(21)=   0.7468414976565969D-01
          TAU(22)=   0.8454459427884981D+00
          WGT(22)=   0.6327404632957483D-01
          TAU(23)=   0.9026378619843071D+00
          WGT(23)=   0.5097582529714779D-01
          TAU(24)=   0.9471590666617142D+00
          WGT(24)=   0.3796238329436276D-01
          TAU(25)=   0.9783854459564710D+00
          WGT(25)=   0.2441785109263180D-01
          TAU(26)=   0.9958857011456169D+00
          WGT(26)=   0.1055137261734301D-01
        ELSE IF(Q.EQ.27)THEN
          TAU(14)=   0.0000000000000000D+00
          WGT(14)=   0.1142208673789570D+00
          TAU(15)=   0.1139725856095300D+00
          WGT(15)=   0.1134763461089651D+00
          TAU(16)=   0.2264593654395369D+00
          WGT(16)=   0.1112524883568452D+00
          TAU(17)=   0.3359939036385089D+00
          WGT(17)=   0.1075782857885332D+00
          TAU(18)=   0.4411482517500269D+00
          WGT(18)=   0.1025016378177458D+00
          TAU(19)=   0.5405515645794569D+00
          WGT(19)=   0.9608872737002846D-01
          TAU(20)=   0.6329079719464952D+00
          WGT(20)=   0.8842315854375694D-01
          TAU(21)=   0.7170134737394237D+00
          WGT(21)=   0.7960486777305775D-01
          TAU(22)=   0.7917716390705082D+00
          WGT(22)=   0.6974882376624561D-01
          TAU(23)=   0.8562079080182945D+00
          WGT(23)=   0.5898353685983363D-01
          TAU(24)=   0.9094823206774911D+00
          WGT(24)=   0.4744941252061501D-01
          TAU(25)=   0.9509005578147051D+00
          WGT(25)=   0.3529705375741962D-01
          TAU(26)=   0.9799234759615012D+00
          WGT(26)=   0.2268623159618064D-01
          TAU(27)=   0.9961792628889886D+00
          WGT(27)=   0.9798996051294237D-02
        ELSE IF(Q.EQ.28)THEN
          TAU(15)=   0.5507928988403427D-01
          WGT(15)=   0.1100470130164752D+00
          TAU(16)=   0.1645692821333808D+00
          WGT(16)=   0.1087111922582942D+00
          TAU(17)=   0.2720616276351780D+00
          WGT(17)=   0.1060557659228464D+00
          TAU(18)=   0.3762515160890787D+00
          WGT(18)=   0.1021129675780607D+00
          TAU(19)=   0.4758742249551183D+00
          WGT(19)=   0.9693065799792995D-01
          TAU(20)=   0.5697204718114017D+00
          WGT(20)=   0.9057174439303280D-01
          TAU(21)=   0.6566510940388650D+00
          WGT(21)=   0.8311341722890114D-01
          TAU(22)=   0.7356108780136318D+00
          WGT(22)=   0.7464621423456874D-01
          TAU(23)=   0.8056413709171791D+00
          WGT(23)=   0.6527292396699963D-01
          TAU(24)=   0.8658925225743951D+00
          WGT(24)=   0.5510734567571671D-01
          TAU(25)=   0.9156330263921321D+00
          WGT(25)=   0.4427293475900418D-01
          TAU(26)=   0.9542592806289382D+00
          WGT(26)=   0.3290142778230441D-01
          TAU(27)=   0.9813031653708728D+00
          WGT(27)=   0.2113211259277126D-01
          TAU(28)=   0.9964424975739544D+00
          WGT(28)=   0.9124282593094647D-02
        ELSE IF(Q.EQ.29)THEN
          TAU(15)=   0.0000000000000000D+00
          WGT(15)=   0.1064793817183143D+00
          TAU(16)=   0.1062782301326792D+00
          WGT(16)=   0.1058761550973209D+00
          TAU(17)=   0.2113522861660011D+00
          WGT(17)=   0.1040733100777293D+00
          TAU(18)=   0.3140316378676399D+00
          WGT(18)=   0.1010912737599150D+00
          TAU(19)=   0.4131528881740086D+00
          WGT(19)=   0.9696383409440859D-01
          TAU(20)=   0.5075929551242276D+00
          WGT(20)=   0.9173775713925879D-01
          TAU(21)=   0.5962817971382278D+00
          WGT(21)=   0.8547225736617256D-01
          TAU(22)=   0.6782145376026866D+00
          WGT(22)=   0.7823832713576377D-01
          TAU(23)=   0.7524628517344771D+00
          WGT(23)=   0.7011793325505127D-01
          TAU(24)=   0.8181854876152524D+00
          WGT(24)=   0.6120309065707916D-01
          TAU(25)=   0.8746378049201028D+00
          WGT(25)=   0.5159482690249790D-01
          TAU(26)=   0.9211802329530588D+00
          WGT(26)=   0.4140206251868294D-01
          TAU(27)=   0.9572855957780877D+00
          WGT(27)=   0.3074049220209369D-01
          TAU(28)=   0.9825455052614132D+00
          WGT(28)=   0.1973208505612272D-01
          TAU(29)=   0.9966794422605966D+00
          WGT(29)=   0.8516903878746441D-02
        ELSE IF(Q.EQ.30)THEN
          TAU(16)=   0.5147184255531770D-01
          WGT(16)=   0.1028526528935589D+00
          TAU(17)=   0.1538699136085835D+00
          WGT(17)=   0.1017623897484055D+00
          TAU(18)=   0.2546369261678899D+00
          WGT(18)=   0.9959342058679524D-01
          TAU(19)=   0.3527047255308781D+00
          WGT(19)=   0.9636873717464425D-01
          TAU(20)=   0.4470337695380892D+00
          WGT(20)=   0.9212252223778609D-01
          TAU(21)=   0.5366241481420199D+00
          WGT(21)=   0.8689978720108300D-01
          TAU(22)=   0.6205261829892429D+00
          WGT(22)=   0.8075589522942016D-01
          TAU(23)=   0.6978504947933158D+00
          WGT(23)=   0.7375597473770522D-01
          TAU(24)=   0.7677774321048262D+00
          WGT(24)=   0.6597422988218053D-01
          TAU(25)=   0.8295657623827684D+00
          WGT(25)=   0.5749315621761907D-01
          TAU(26)=   0.8825605357920526D+00
          WGT(26)=   0.4840267283059396D-01
          TAU(27)=   0.9262000474292743D+00
          WGT(27)=   0.3879919256962706D-01
          TAU(28)=   0.9600218649683075D+00
          WGT(28)=   0.2878470788332336D-01
          TAU(29)=   0.9836681232797472D+00
          WGT(29)=   0.1846646831109102D-01
          TAU(30)=   0.9968934840746495D+00
          WGT(30)=   0.7968192496166118D-02
        ELSE
          IRET=1
          GO TO 999
        END IF


C
C ARGUMENT VALUES AND WEIGHTS (FIRST HALF)
        QOLD=Q
        IQ=Q/2
        DO 10 I=1,IQ
          WGT(I)=WGT(Q+1-I)
          TAU(I)=-TAU(Q+1-I)
10      CONTINUE
C
C SET COEFFICIENTS MATRIX
        DO I=1,Q
          DO K=1,Q
            IK = I+(K-1)*Q
            MAT(IK)=K*TAU(I)**(K-1)
          ENDDO
        ENDDO
C
C INVERT MATRIX
        CALL DMINV(MAT,Q,DET,LHLP,MHLP)
      END IF

C
C COMPUTATION OF THE DEFINITE INTEGRAL
C ------------------------------------
      T0=(TA+TB)/2
      H =(TB-TA)/2
      DO KDIM=1,IDIM
        VALUE(KDIM)=0.D0
        COEF(KDIM,1)=0.D0
      ENDDO

C
C DEFINITE INTEGRAL
C -----------------
      DO I=1,Q
        TEXT= T0+TAU(I)*H
        I0=(I-1)*IDIM
        IF (PRESENT(satPos)) THEN
          CALL deriv(text,rhs(i0+1), satpos, stdorb, act_sat)
        ELSE
          CALL deriv(text,rhs(i0+1))
        ENDIF
        DO KDIM=1,IDIM
          VALUE(KDIM)=VALUE(KDIM)+WGT(I)*RHS(I0+KDIM)
        ENDDO
      ENDDO
      DO KDIM=1,IDIM
        VALUE(KDIM)=VALUE(KDIM)*H
      ENDDO
C
C COEFFICIENTS FOR POLYNOMIAL SOLUTION
C ------------------------------------
      DO I=1,Q
        DO KDIM=1,IDIM
          COEF(KDIM,I+1)=0.D0
          DO K=1,Q
            IK = I+(K-1)*Q
            K0=(K-1)*IDIM
            COEF(KDIM,I+1)=COEF(KDIM,I+1)+MAT(IK)*RHS(K0+KDIM)*H
          ENDDO
        ENDDO
      ENDDO
C
C COMPUTE INITIAL CONDITIONS FOR CENTER OF INTERVAL
      DT=(TA-T0)
      CALL POLEVN(1,Q,IDIM,DT,H,COEF,RHS)
      DO KDIM=1,IDIM
        COEF(KDIM,1)=VALINI(KDIM)-RHS(KDIM)
      ENDDO

999   CONTINUE
      RETURN
      END SUBROUTINE

      END MODULE
