MODULE f_vmf1ell

PRIVATE:: ellGrd

CONTAINS


! -------------------------------------------------------------------------
! Bernese GPS Software Version 5.1
! -------------------------------------------------------------------------

FUNCTION vmf1ell(key,zpd,xstell)

! -------------------------------------------------------------------------
! Purpose:    Correct the zenith path delays provided by the VMF1 grids
!             from the http://www.hg.tuwien.ac.at/%7Eecmwf1/GRID/orography_ell
!             to the height of the station
!             Kouba: Implementation and testing of the gridded Vienna Mapping
!             Function 1 (VMF1), J.Geod(2008) 82:193-205
!
! Author:     R. Dach
!
! Created:    30-Jun-2008
!
! Changes:    20-Sep-2010 RD: Modify assignment of the height model
!             11-Oct-2010 RD: Accelerate the assignement of the array
!             14-Oct-2010 RD: Correct/simplify access to the array
!             20-Sep-2012 RD: Use M_BERN with ONLY
!             15-Oct-2012 RD: Put the grid in a local SR with "CASE"
!
! Copyright:  Astronomical Institute
!             University of Bern
!             Switzerland
! -------------------------------------------------------------------------

  USE m_bern,  ONLY: i4b, r8b, lfnerr
  USE d_const, ONLY: rho

  USE s_exitrc
  IMPLICIT NONE

! List of Parameters
! ------------------
  ! input
  CHARACTER(LEN=*)                        :: key    !'ZH'/'ZW' to distinguish
                                                    ! between both ZPD
  REAL(r8b)                               :: zpd    ! Zenith path delay from
                                                    ! grid (meters)
  REAL(r8b), DIMENSION(3)                 :: xStell ! Station coordinates
                                                    ! 1,2: lat,lon in radiant
                                                    ! 3:   hgt in meter
  ! output
  REAL(r8b)                               :: vmf1ell ! corrected ZPD

! Local Parameters
! ----------------
  CHARACTER(LEN=7),        PARAMETER      :: srName = 'vmf1ell'
! Step length in the grid
  REAL(r8b), DIMENSION(2), PARAMETER      :: dGrid = (/ 2.0d0,2.5d0 /)

! Local Variables
! ---------------

  REAL(r8b)                               :: bSta, bMin
  REAL(r8b)                               :: lSta, lMin
  REAL(r8b)                               :: hSta, hMin, hMax

  INTEGER(i4b)                            :: iLat,jLat
  INTEGER(i4b)                            :: iLon,jLon

! Grid from http://www.hg.tuwien.ac.at/%7Eecmwf1/GRID/orography_ell
! -----------------------------------------------------------------
 ! IF (first) THEN
 !   first = .FALSE.

!    ALLOCATE(ellGrd(145,91),stat=iac)
!    CALL alcerr(iac,'ellGrd',(/145,91/),srName)


 ! ENDIF

! Convert the station coordinates from radiant to degree
! ------------------------------------------------------
  bSta = 90d0 - xStell(1) * rho
  lSta = xStell(2) * rho
  IF (lSta < 0D0) lSta = lSta + 360d0

! Find the Grid coordinates
! -------------------------
  bMin = int( bSta / dGrid(1) ) * dGrid(1)
  lMin = int( lSta / dGrid(2) ) * dGrid(2)

! Get the index in the grid
! -------------------------
  iLat = NINT( bMin / dGrid(1) ) + 1
  iLon = NINT( lMin / dGrid(2) ) + 1

  jLat = iLat+1
  jLon = iLon+1

! Interpolate the height in the grid
! ----------------------------------
  hMin = DBLE(ellGrd(iLon,iLat)) + DBLE( ellGrd(iLon,jLat) - ellGrd(iLon,iLat) ) * &
         DABS(bSta - bMin) / dGrid(1)

  hMax = DBLE(ellGrd(jLon,iLat)) + DBLE( ellGrd(jLon,jLat) - ellGrd(jLon,iLat) ) * &
         DABS(bSta - bMin) / dGrid(1)

  hSta = hMin + (hMax - hMin) * (lSta - lMin) / dGrid(2)


! Correction for hydrostatic ZPD
! ------------------------------
  IF ( key == 'ZH' ) THEN
    vmf1ell = zpd / (1D0 - 2.26D-5 * hSta )**5.225D0 &
                  * (1D0 - 2.26D-5 * xStell(3) )**5.225D0 &
                  * (1D0 - 0.00266D0*DCOS(2D0*xStell(1)) - 0.28D-6*hSta) &
                  / (1D0 - 0.00266D0*DCOS(2D0*xStell(1)) - 0.28D-6*xStell(3))

! Correction for wet ZPD
! ----------------------
  ELSE IF ( key == 'ZW' ) THEN
    vmf1ell = zpd * exp(-(xStell(3)-hSta)/2D3)

! Return directly the height from the grid
! ----------------------------------------
  ELSE IF ( key == 'HGT' ) THEN
    vmf1ell = hSta

! Unknown key
! -----------
  ELSE
    WRITE(lfnerr,'(/,3A,/)') &
    ' *** SR VMF1ELL: Key "',TRIM(key),'" is not supported.'
    CALL exitrc(2)
  ENDIF

END FUNCTION vmf1ell

FUNCTION ellGrd(ii,jj)
USE m_bern, ONLY: i4b
IMPLICIT NONE
  INTEGER(i4b) :: ii,jj,ellGrd

  INTEGER(i4b), DIMENSION(145) :: ellGrdHlp
  SELECT CASE(jj)
  CASE( 1)
  ellGrdHlp(1:145) = (/ &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9,      9,      9,      9,      9,      9, &
    9,      9,      9,      9,      9 /)

  CASE( 2)
  ellGrdHlp(1:145) = (/ &
   18,     19,     20,     21,     22,     22,     23,     23,     22,     21, &
   20,     19,     19,     17,     15,     14,     13,     10,      9,      9, &
   10,     10,     11,     10,     10,     11,     12,     12,     13,     14, &
   13,     13,     13,     13,     13,     13,     12,     12,     12,     12, &
   12,     12,     12,      9,      8,      7,      5,      5,      3,      2, &
    0,     -0,     -0,     -1,     -0,      1,      5,      6,      7,      9, &
   10,     14,     19,     24,     29,     34,     38,     33,     29,     24, &
   19,     15,     10,      6,      1,     -4,     -8,    -13,    -17,    -11, &
   -3,      7,     15,     23,     32,     33,     33,     34,     33,     33, &
   34,     28,     22,     16,     10,      3,     -4,     -4,     -4,     -4, &
   -4,     -4,     -3,     -1,      2,      4,      6,     10,     12,     13, &
   16,     17,     19,     20,     22,     22,     21,     20,     20,     20, &
   19,     17,     16,     14,     13,     11,     10,      9,     10,     10, &
   11,     12,     12,     13,     13,     14,     14,     15,     15,     16, &
   16,     17,     17,     18,     18 /)

  CASE( 3)
  ellGrdHlp(1:145) = (/ &
   45,     36,     27,     17,     14,     13,     12,     18,     26,     31, &
   28,     24,     21,     17,     15,     16,     18,     21,     22,     21, &
   20,     16,     13,     11,     11,     12,     13,     14,     13,     14, &
   16,     16,     17,     19,     19,     19,     17,     14,     13,     14, &
   15,     18,     22,     23,     22,     20,     15,      7,     -3,     -4, &
   -4,     -5,     -1,      4,     10,     13,     11,     10,      9,      9, &
    8,      9,     13,     17,     19,     16,     13,     10,      5,     -0, &
   -4,     -3,     -1,      5,     10,     17,     22,     26,     22,     14, &
    7,      2,     -2,     -6,     -1,     11,     21,     26,     25,     25, &
   20,     13,      6,      7,     16,     25,     33,     30,     26,     18, &
    2,    -13,    -21,    -19,     -8,      4,     19,     33,     41,     46, &
   41,     36,     30,     26,     24,     21,     22,     24,     26,     26, &
   26,     26,     27,     26,     23,     21,     18,     15,      9,      7, &
    7,     10,     14,     17,     21,     22,     17,     11,      6,      8, &
   11,     15,     24,     35,     45 /)

  CASE( 4)
  ellGrdHlp(1:145) = (/ &
   38,     41,     43,     22,      1,     13,     26,     36,     45,     25, &
    4,      5,      9,     31,     54,     42,     30,     11,     -9,      4, &
   18,     26,     35,     20,      4,      1,     -1,      9,     20,     17, &
   15,     12,     10,     12,     16,     11,      6,      3,     -0,     10, &
   20,     26,     33,     26,     20,     18,     17,     20,     22,     15, &
   10,      3,     -4,     -1,      2,      4,      6,      2,     -3,      1, &
    3,      9,     13,      8,      3,      2,      2,      5,      8,      1, &
   -6,     -7,     -8,      5,     18,     20,     22,     12,      4,      5, &
    7,     13,     18,     10,      2,      0,     -1,      9,     20,     15, &
   11,      6,      1,     11,     21,     15,     10,     -4,    -16,     -4, &
   10,     16,     23,      8,     -9,      3,     13,     49,     86,     98, &
  110,     90,     71,     39,      7,    -23,    -53,    -47,    -39,      7, &
   53,     64,     74,     35,     -3,    -11,    -20,     30,     80,    128, &
  177,    155,    131,     61,     -6,    -15,    -24,     17,     55,     45, &
   34,     12,     -9,     14,     38 /)

  CASE( 5)
  ellGrdHlp(1:145) = (/ &
   54,     36,     21,      7,     29,     46,     53,     33,     16,     17, &
   30,     40,     36,     25,     13,     10,     20,     32,     36,     21, &
    6,     -2,     19,     35,     45,     27,     12,      3,      9,     12, &
   11,     12,     14,     17,      8,     -4,    -10,     -6,     -6,    -10, &
  -10,     -4,      7,     10,      9,      5,     12,     14,     11,      3, &
    1,      9,     13,     10,     -3,     -8,     -4,      6,      8,      2, &
   -6,     -5,      2,      9,      8,      4,      1,      3,      1,     -2, &
   -4,      0,     10,     10,      6,     -2,      2,     10,     18,     12, &
    2,     -6,     -2,      4,      5,      2,     -1,      1,      3,      4, &
    3,      6,      5,      1,     -4,     -3,      3,      2,     -0,     -2, &
    5,      4,     -4,      0,     22,     48,     58,     57,    173,    756, &
 1154,   1559,   1846,   1957,   1933,   1767,   1440,   1045,    775,    421, &
  333,    338,    340,    346,    621,    701,    955,   1164,   1280,   1222, &
 1061,    884,    703,    571,    474,    212,    194,    193,     95,     98, &
   64,     13,     15,     30,     54 /)

  CASE( 6)
  ellGrdHlp(1:145) = (/ &
   28,     32,     25,     51,    117,    201,    250,    308,    370,    409, &
  398,    194,     37,     16,     23,     17,      8,     21,     52,     49, &
   30,     30,     55,     93,    107,     67,     18,     -9,    -10,      4, &
    9,     10,     21,     17,     -9,    -33,     18,    241,    345,    364, &
  150,    -22,    -80,    -23,     13,     16,      4,     -5,     -3,      4, &
   11,     15,     11,     -1,    -11,     -3,      2,     -2,     -7,     -1, &
    8,     -3,     -7,      3,     14,     13,      5,      6,     -0,     -9, &
   -8,      9,     17,     -1,     -4,      7,     17,     12,     -0,      7, &
    4,    -11,    -21,    -12,     10,      1,     -9,     -1,     10,      8, &
  -14,    -10,      4,     -1,    -19,    -28,     -2,     22,      9,    -12, &
  -10,     27,     50,    -40,    -35,    259,    606,   1235,   1203,    677, &
  492,    780,   1294,   1647,   1565,   1134,    432,    269,    195,    646, &
 1119,   1406,   1634,   1858,   2054,   2179,   2230,   2233,   2201,   2142, &
 2035,   1843,   1521,   1150,    893,    732,    639,    350,     71,    -36, &
   -4,     46,     55,     32,     28 /)

  CASE( 7)
  ellGrdHlp(1:145) = (/ &
   33,     25,     57,     26,     58,    268,    655,    622,    347,    223, &
  121,     37,     13,     38,     25,     -5,      3,     19,     14,     11, &
    8,     -6,      1,     28,     32,      4,    -16,    -11,     -7,    -13, &
   -5,     19,     20,    -11,    -24,     -1,     14,     -1,    -20,     -1, &
   42,    104,     87,      9,    -33,     15,     24,    -23,    -26,     13, &
   11,    -10,     -4,      7,      1,     -3,      2,      9,      5,      1, &
    4,     -0,     -4,      3,     10,     10,     13,     11,      0,     -1, &
    5,      3,     -4,      1,      8,      1,     -3,      8,     17,      4, &
  -19,    -16,      5,      3,    -11,     -1,     11,     -1,     -8,     -3, &
   -9,     -6,     11,     -7,    -31,      0,     24,     -5,      6,     48, &
   30,     -3,     -3,     -5,     37,     86,    110,    132,    199,    300, &
  427,    890,   1222,    679,    203,    298,    960,   1332,   1524,   1811, &
 2049,   2141,   2280,   2448,   2476,   2489,   2597,   2586,   2429,   2370, &
 2294,   2022,   1774,   1655,   1347,    756,    209,     71,     37,     28, &
   29,     30,     45,     61,     33 /)

  CASE( 8)
  ellGrdHlp(1:145) = (/ &
   74,     24,     28,     50,     33,     23,     41,     35,     29,     38, &
   24,     34,     45,     23,     18,      7,    -10,     18,     12,    -13, &
   20,     -8,    -18,    144,    462,    466,    367,    165,      4,    -36, &
   11,     14,     -9,     -5,    -12,    -20,     20,     82,    143,    108, &
   96,    196,    376,    546,    522,    240,      6,    -22,     -2,     -9, &
   -5,      3,     -5,    -11,      1,     13,     18,     25,     17,      9, &
    6,     -1,     -5,     -4,      5,     16,      9,      1,      9,      8, &
   -6,     -6,      3,      2,     -2,     -6,     -4,     11,      8,    -14, &
   -9,    -10,    -13,     10,      8,     -3,      7,      1,    -15,    -15, &
   -4,      9,     -2,    -20,     -4,     20,     63,    111,    234,    217, &
   71,     11,     17,     58,    133,    140,    151,    257,    245,    153, &
  356,    602,    420,    110,     14,     39,     93,    220,    283,    353, &
  960,   1521,   1968,   2184,   2391,   2627,   2771,   2838,   2868,   2797, &
 2673,   2603,   2548,   2364,   1926,   1202,    518,    113,     23,     34, &
   66,     42,     18,     64,     74 /)

  CASE( 9)
  ellGrdHlp(1:145) = (/ &
   48,     45,     39,     45,     58,     36,     29,     35,     30,     48, &
   29,     16,     40,     20,     13,      4,      8,     16,     -4,     34, &
  -26,     32,    463,    329,     12,    -51,     34,     17,    -21,    -14, &
    2,     -3,    -10,     11,     21,    101,    197,    263,    311,    251, &
  148,    148,    124,     92,     32,     -4,      6,      4,    -19,    -14, &
   -7,     -1,     -1,    -11,    -10,      9,     17,     19,     16,     -4, &
    7,      6,     -5,     -4,    -10,      3,    -10,     -7,     13,     -2, &
    5,    -16,    -11,     21,    -13,     -3,     11,     -1,      6,    -10, &
   -7,     -2,      2,     19,     -3,     -5,     -8,    -22,    -14,    -28, &
   -9,      6,     -7,     -9,     21,    194,    267,    106,    -29,    -13, &
  -36,    -16,    -11,    -20,     17,     32,    100,    179,    110,    148, &
  149,    185,    237,    120,     -9,     -7,      7,    -25,     13,     18, &
  -38,    178,   1198,   2069,   2487,   2734,   2912,   2973,   3059,   3076, &
 2964,   2862,   2572,   2005,   1512,    944,    263,     36,     53,     50, &
   52,     47,     45,     55,     48 /)

  CASE(10)
  ellGrdHlp(1:145) = (/ &
   59,     51,     41,     51,     47,     56,     20,     33,     44,     32, &
   29,      4,     15,     22,     16,      2,     15,     13,     -0,     14, &
   -8,    153,    251,    -15,    -16,      8,    -22,     48,     41,     28, &
   22,     29,     27,     46,     60,     70,     86,     85,     94,     58, &
   60,    103,    185,    243,    195,    136,     82,     71,     86,    170, &
  240,    185,     32,    -16,      9,     11,     70,     59,     52,     53, &
   21,      4,     -1,     -3,     -2,     -7,    -11,    -10,     -2,     18, &
   -1,    -15,     32,     18,    -13,      6,     -9,     20,    -11,      1, &
    1,      0,     25,      7,      4,    -21,    -14,    -28,    -25,    -22, &
    3,     -7,     26,    -25,     75,    468,    198,    177,    275,    269, &
  168,     97,     73,    -57,     19,     39,    108,     12,     58,    181, &
  200,    358,    359,    740,    678,    107,     -2,    -11,     49,    -19, &
   41,     74,    809,   1585,   2187,   2550,   2782,   2997,   3125,   3241, &
 3156,   2991,   2307,   1944,   1522,    259,     42,     67,     45,     59, &
   57,     41,     65,     45,     59 /)

  CASE(11)
  ellGrdHlp(1:145) = (/ &
   37,     44,     44,     52,     61,     38,     18,     60,    340,    692, &
  540,    396,    211,     82,     23,      7,     22,     23,     -9,     20, &
    9,     -4,     -7,     14,     24,     53,      9,     34,     68,     22, &
   59,     68,     41,     48,     73,    169,    567,    978,    889,    705, &
  548,    409,    487,    680,    554,    292,    131,    186,    245,    243, &
  111,    648,    745,    342,    419,    445,    471,    287,    129,    158, &
  268,    174,    106,    112,     24,     70,     58,    -23,    100,    219, &
   62,     48,    -29,      8,     -8,      9,     -5,     12,    -29,     -1, &
  159,    143,    113,    107,     65,    203,    401,    245,    121,    -52, &
    6,    -27,    233,    465,    -11,     81,     43,    -22,    146,    244, &
  297,    159,     98,     37,      0,    -36,    175,    178,    -32,      9, &
  114,    108,     44,    257,    707,    960,    962,    321,     29,     -2, &
   41,      5,    643,    965,   1301,   2075,   2441,   2801,   2978,   3218, &
 3090,   2868,   2367,   1963,   1219,    311,     91,     55,     70,     50, &
   56,     51,     42,     60,     37 /)

  CASE(12)
  ellGrdHlp(1:145) = (/ &
   37,     43,     45,     49,     51,     47,    251,   1123,    824,    478, &
  320,    247,    381,    408,    340,    251,     93,     18,     72,     21, &
   76,     42,    103,     91,    162,     88,    452,    388,     37,     32, &
   12,     41,     48,     80,     95,    226,    561,    830,    963,    925, &
  735,    391,    389,    360,    369,    398,    305,    279,    280,    164, &
  308,   1286,   1159,    544,    515,   1048,   1071,    389,    135,    210, &
  231,     83,    136,    214,    248,    309,    566,    875,    634,    543, &
  785,   1024,    751,    225,      5,     -5,     -1,     15,    201,    590, &
  791,    848,   1275,   1331,   1411,   1363,   1057,    837,    662,    794, &
  419,    190,    230,    263,    282,    304,    392,    353,    109,     15, &
    7,     74,    157,     57,     21,     -5,     40,    192,    295,    108, &
  332,    245,    -24,     11,    -15,    195,    335,    769,    297,     -7, &
   32,     16,     99,    256,    964,   1941,   2342,   2817,   2929,   3038, &
 2557,   1253,    723,    214,     11,     82,     99,     54,     66,     63, &
   47,     66,     39,     63,     37 /)

  CASE(13)
  ellGrdHlp(1:145) = (/ &
   62,     15,     66,     40,      6,    328,   1020,    683,    469,    180, &
   72,    259,    312,    127,     35,     59,     79,    114,     67,    112, &
  155,     77,    118,     76,    232,    472,    293,     -5,     92,     15, &
   68,     44,     41,     38,    110,    107,    357,    567,    731,    709, &
  632,    652,    559,    650,    691,    564,    445,    301,    214,    145, &
  241,   1153,   1261,    730,   1012,   1252,   1783,   1251,   1302,   1139, &
  287,    388,    683,    519,    649,    802,   1009,    877,    611,    458, &
  392,    336,    296,    305,    356,    425,    113,     35,    220,    236, &
  182,    333,    397,    512,    691,    487,    530,    395,   1239,    804, &
  708,    515,    267,    230,    358,    192,     54,    316,    449,    400, &
  433,    324,    305,    324,    202,    112,    261,    302,    339,    197, &
   64,    -19,    -16,    -36,      3,    105,    206,     18,    675,    768, &
  -90,    104,    -25,    904,   1422,   2218,   2719,   2541,   1600,    898, &
  338,    -44,    101,    -20,    120,    285,    362,    558,    291,     94, &
   39,     66,     51,     52,     62 /)

  CASE(14)
  ellGrdHlp(1:145) = (/ &
   52,     30,     63,     32,    240,    737,    571,    442,    222,     21, &
   60,     52,    165,    228,    122,    143,     92,    107,    154,    160, &
  233,    160,    153,    271,    552,     55,    151,     73,     72,     79, &
   75,     72,     77,     84,     96,     59,    366,    492,    493,    592, &
  544,    514,    525,    477,    476,    456,    383,    213,    176,    190, &
  109,    474,    771,   1082,   1162,   1263,   1458,   1307,   1488,   1446, &
  957,    548,    679,    995,   1043,    886,    855,    550,    320,    466, &
  364,    161,     -2,     14,     29,     54,     17,    -10,     57,    -20, &
  413,    253,    489,    450,    865,   1306,   1330,   1103,   1206,   1263, &
 1507,   1631,   1806,   1580,    591,    467,    366,    344,    367,    392, &
  416,    352,    323,    344,    191,    130,     92,     49,     25,    -19, &
   85,     51,    -49,     27,    -29,    126,    378,    606,    445,    -31, &
   29,      4,     22,    133,   1104,   2257,   2769,   2197,    223,     66, &
   44,     41,     63,     40,     80,    155,    627,    875,    361,     18, &
   79,     55,     47,     63,     52 /)

  CASE(15)
  ellGrdHlp(1:145) = (/ &
   65,     32,    162,   1259,   1271,    913,    566,    123,     13,     72, &
  110,    126,     79,    140,     93,     99,    179,    141,    162,    169, &
  161,    126,    183,    248,    481,     95,    135,    105,     69,     71, &
   75,     60,     59,    100,    106,     95,    144,    429,    599,    496, &
  491,    504,    539,    429,    429,    455,    385,    211,    293,    446, &
  283,    281,    240,    313,    313,   1143,   1744,   1840,   1553,   1246, &
 1186,    791,   1015,    937,    572,    389,    301,    455,   1168,    879, &
  242,     47,     41,    -20,      6,      8,     -1,      7,     82,    187, &
  228,    449,    758,   1638,    707,   1646,   1419,   2335,   1472,   1283, &
 1217,   1470,   1535,   1885,   1489,    421,    532,    255,    142,    204, &
  331,    418,    407,    355,    273,    179,     68,    -46,    -70,    -56, &
  -60,    -25,    -54,    146,    359,    151,    -42,    151,     19,      5, &
    4,      1,     30,     21,    194,   1817,   2554,    587,     42,     23, &
   69,     45,     53,     69,     67,     91,     41,     86,     90,     51, &
   58,    121,     77,     45,     65 /)

  CASE(16)
  ellGrdHlp(1:145) = (/ &
   41,     66,    122,   1171,    650,    304,    318,     81,     28,     29, &
   20,     38,    100,     53,    195,    153,    178,    179,    207,    173, &
  182,    150,    154,    355,    401,     89,     65,     65,     87,     82, &
   48,     71,     54,     51,     97,     99,    141,    564,    550,    463, &
  372,    403,    506,    474,    513,    438,    735,    561,    377,    561, &
  522,    545,    416,    373,    353,   1056,    952,    697,    676,    546, &
  693,    837,    169,     55,     27,    466,    159,     89,     95,    -16, &
   16,    -22,     -2,      4,      0,      8,      5,     27,     48,     51, &
  669,    313,    560,    311,    795,     19,    286,    472,   1622,   1605, &
 1528,   1376,   1447,    974,   1015,    575,    620,    515,    465,    255, &
  319,    473,    428,    418,    305,    219,     -1,    -48,    -51,    -46, &
  -48,    -53,    -29,      2,    190,    253,    129,    -31,    141,      7, &
   31,     -9,     31,     22,     31,      7,    328,    200,     86,     34, &
   46,     74,     74,     57,     63,     73,     34,     73,     65,     49, &
   52,     58,     55,     59,     41 /)

  CASE(17)
  ellGrdHlp(1:145) = (/ &
   49,     57,     55,    166,      1,    179,    285,     34,     32,     28, &
   76,    115,     93,    157,    192,    168,    149,    166,    144,    127, &
  174,    178,    192,    259,    291,    141,     66,     47,     62,     88, &
   51,     63,     61,     66,     57,     87,    168,    211,    294,    320, &
  346,    392,    555,    489,    652,    869,   1025,   1206,    983,    846, &
 1005,   1131,   1172,    896,    674,    919,    663,     -6,     32,     22, &
   29,     14,    -37,     53,    825,    359,     81,    -15,     15,     -1, &
   -1,    -14,      8,      9,    -16,      6,     22,     12,     21,     20, &
  -27,    214,    741,    130,     51,     13,     19,     46,     31,    169, &
  462,   1569,   1547,   1790,   2053,    888,    742,    524,    491,    522, &
  409,    547,    471,    406,    330,    259,    149,    -15,    -27,    -45, &
  -43,    -49,    -51,     21,    261,    287,    239,    151,    668,    485, &
   -4,     11,      8,     20,     46,     18,     47,     66,     36,     65, &
   57,     53,     76,     77,     39,     56,     56,     71,     54,     48, &
   52,     68,    358,     97,     49 /)

  CASE(18)
  ellGrdHlp(1:145) = (/ &
   40,     42,     42,     53,    101,     92,     74,     27,     25,    137, &
  154,    179,    176,    243,    235,    205,    165,    146,    131,    128, &
  151,    176,    156,    283,    425,    180,     85,     95,     80,     81, &
   52,     87,    106,    106,    166,    307,    281,    464,    413,    399, &
  460,    482,    765,    852,   1575,   1685,   1553,   1597,   1318,   1023, &
 1092,   1244,   1523,   1464,   1166,    655,    -32,     15,     51,     26, &
   24,     -9,    -19,    863,   1266,    259,     43,     10,     21,    -17, &
   -4,      2,     12,     10,      0,     -1,     17,     27,    -10,     67, &
  280,     77,     50,    -17,     51,      6,     -7,     23,    -49,    -28, &
   64,    480,   1229,   1399,   1472,   1284,    781,    637,    682,    631, &
  621,    445,    450,    434,    286,    231,    146,     69,    123,    111, &
  -55,    -45,    -29,     13,    294,    317,    496,    489,    482,    500, &
   -5,      4,     12,     21,     31,     24,     37,     46,     45,     63, &
   49,     64,     66,     60,     60,     68,     40,     70,     73,     53, &
   35,     50,    415,    115,     40 /)

  CASE(19)
  ellGrdHlp(1:145) = (/ &
   68,     40,     41,     53,     84,     80,     46,    168,    167,    186, &
  207,    230,    165,    204,    225,    236,    158,    173,    217,    212, &
  137,    219,    236,    642,    423,    181,    141,    157,    133,    123, &
   50,     79,    137,    177,    282,    578,   1002,    550,   1471,   1828, &
 1002,    519,    854,    961,   1324,   1402,   1125,   1207,    999,    807, &
  729,    589,    665,    796,   1103,    170,    114,     39,      9,     14, &
   17,      3,     33,   1086,    354,    -22,     -6,     10,     13,      8, &
    3,    -11,      8,     10,      8,     10,      1,     55,     63,      9, &
    5,      5,     11,     -1,     30,     24,    -22,    -13,    -10,    -11, &
  -10,     52,    620,   1408,   1058,   1006,   1952,   1277,    851,    632, &
  631,    582,    564,    347,    252,    199,    232,    203,    177,    115, &
  125,     -4,    -46,    132,    292,    475,    550,    655,    551,    508, &
  343,    202,     12,     14,     32,     32,     30,     37,     47,     56, &
   54,     67,     62,     60,     57,     58,     60,     71,     45,     60, &
  136,    235,     45,    312,     68 /)

  CASE(20)
  ellGrdHlp(1:145) = (/ &
  185,     43,     48,    191,    364,    114,    116,    132,    187,    167, &
  175,    202,    153,    157,    192,    232,    152,    159,    257,    140, &
   78,    170,    200,    428,    305,    186,    200,    305,    288,    288, &
  186,    134,    123,    319,    699,   1173,   1914,   1729,   1562,   1928, &
 2420,   1781,    821,    962,   1125,   1139,    973,    954,    920,    994, &
  700,    352,    297,    981,   1237,    594,    332,    186,    -36,    -15, &
   15,     25,     31,    535,    -36,     -8,     18,      7,     -7,     -4, &
   15,      5,     33,     31,     41,     13,      2,     -8,      2,      0, &
   16,     -9,     17,      6,     -2,     12,     -8,    -14,      2,    -14, &
  -17,      6,     -9,    871,   1938,   1203,   1790,   2551,   1608,    863, &
  734,    598,    580,    595,    399,    225,    281,    347,    259,    238, &
  114,     73,    -43,    149,    394,    620,    650,    674,    637,    554, &
  441,    359,     13,     13,     33,     33,     25,     31,     46,     51, &
   55,     65,     57,     64,     61,     58,     60,     63,     46,     89, &
  242,    160,     60,    299,    185 /)

  CASE(21)
  ellGrdHlp(1:145) = (/ &
   59,    224,    359,    457,    454,    698,    516,    718,    701,    368, &
  257,    278,    255,    166,    166,    184,    177,    145,    151,     24, &
  -30,     52,    170,    337,    283,    151,    169,    411,    391,    553, &
  686,    631,    347,    755,   1750,   2691,   2654,   1480,   1673,   2215, &
 2105,   1592,   1235,   1152,   1593,   1356,    813,    801,    795,    928, &
  394,    349,    313,    689,    942,    441,    596,    517,    -25,     -5, &
   -3,     -1,     49,     -2,     13,     27,      5,     -5,      1,      4, &
    2,     -9,      2,     -3,      4,      0,     -5,     -1,      3,      7, &
   21,     -2,      7,     13,     -4,      4,      6,     -7,     -3,    -13, &
  -16,    -15,    -43,    288,    937,   1749,   1464,   1962,   2268,    829, &
  916,    845,    664,    607,    511,    236,    304,    399,    426,    313, &
  211,    149,    166,    187,    355,    381,    578,    377,     66,     42, &
   26,    357,      9,      0,     11,     18,     36,     48,     52,     49, &
   50,     53,     52,     63,     65,     67,     63,     61,     52,     74, &
   49,     77,     89,     66,     59 /)

  CASE(22)
  ellGrdHlp(1:145) = (/ &
  239,    284,    378,    794,    898,    987,   1114,    298,    461,    307, &
 1178,    231,    217,    170,    103,    244,    209,    106,     51,     60, &
  -13,     21,    100,    227,    220,    113,    219,    511,    424,    770, &
  717,    610,    666,   1058,    880,   1491,   3066,   1130,   1802,   2782, &
 2483,   1614,   1466,   1815,   1440,   1175,    918,    665,   1055,    649, &
  272,    364,    667,    145,    216,   1077,    188,    207,     33,     32, &
   20,     43,     11,     -1,     -1,     -1,    -18,     -4,      2,     -8, &
   -4,     -8,     -2,    -14,      2,     -5,      8,     15,      9,     -2, &
   27,     -6,     -3,     11,    -13,    -13,      7,     -4,     -1,    -17, &
  -27,    -22,    -51,     45,     96,    336,   1342,   1068,   1471,   1745, &
 1099,    861,    812,    662,    531,    346,    411,    469,    353,     80, &
  350,    349,    330,    328,    418,    436,    540,    417,    116,    -15, &
  -12,    390,    249,     26,     13,     24,     33,     28,     35,     42, &
   56,     66,     62,     66,     62,     60,     50,     56,     62,     87, &
   31,     68,     68,    210,    239 /)

  CASE(23)
  ellGrdHlp(1:145) = (/ &
  162,    744,    604,   2463,   1927,   1274,    803,    289,    149,   1032, &
 1107,    382,    102,     31,     47,     36,    112,    203,     48,     14, &
  -71,    -20,     33,    140,      2,     71,    125,    181,    335,    459, &
  253,    352,    879,   1258,   1161,    535,   1410,   2364,   2218,   2920, &
 1923,   1924,   1437,   1390,   1132,    992,   1146,   1096,   1033,    296, &
  214,    365,    599,    252,    556,    459,     43,     97,     33,     32, &
   89,     14,     17,     15,     10,      7,     -2,      2,     -4,    -15, &
   -8,     -5,      0,     -8,      3,     -6,      3,      1,      1,    -12, &
    6,     -4,     -6,     -3,    -12,    -18,    -10,    -23,    -29,    -36, &
  -37,    -27,    -44,    -30,    -33,    767,    500,   1214,   2016,   2056, &
 1804,   1139,   1030,    871,    615,    403,    426,    365,    461,    311, &
  195,    133,    194,    285,    270,    281,    478,    250,    126,     50, &
   44,      2,     -3,     10,     18,     29,     34,     36,     43,     48, &
   52,     56,     60,     66,     66,     70,     63,     61,     59,     59, &
   26,     52,     18,     34,    162 /)

  CASE(24)
  ellGrdHlp(1:145) = (/ &
  170,    663,    719,   1207,    688,    475,    250,   1439,    863,    660, &
  100,    212,     48,     61,     72,     61,   1293,   1637,     -6,    -10, &
 -114,    169,    215,    171,    -22,     70,    123,    426,    451,    420, &
  704,   1061,   1905,   2116,   2529,   1767,   1591,   1999,   1281,   1553, &
 1666,   1723,   1479,   1223,   1054,   1160,   1160,   1419,    571,    273, &
  293,    675,    862,    473,    662,     -3,     -6,    686,    201,     47, &
    6,     14,     17,     14,      5,      3,     -8,     -5,    -12,    -18, &
   -6,     -9,     -4,    -13,      2,    -12,      2,     -6,     10,    -12, &
   -3,     -6,    -12,    -13,    -13,    -23,    -18,    -38,    -36,    -38, &
  -36,    -11,    -19,    -12,    -10,   1173,   1477,   1339,   2584,   2118, &
 3021,   2137,   1439,    963,    633,    488,    388,    341,    307,    184, &
  345,     82,    300,    140,    545,    722,    244,    -11,     78,    -27, &
  -16,    -14,    -14,      4,     12,     18,     24,     32,     40,     46, &
   47,     53,     63,     66,     63,     68,     62,     63,     58,     52, &
   28,    175,     71,     64,    170 /)

  CASE(25)
  ellGrdHlp(1:145) = (/ &
 1028,    887,     62,      5,    272,    592,    593,     19,   1437,   1268, &
 1044,    352,     14,    290,    408,   -111,   -201,   1570,   1894,   2346, &
  -98,     24,    180,     80,     68,    130,    302,    232,   2239,   2988, &
 3089,   3670,   4398,   2462,   1916,   1962,    950,    821,   1647,   1548, &
 1154,   1210,   1218,   1320,   1372,   1424,   1439,   1506,    916,    192, &
  579,   1178,    811,     33,     10,     20,    326,    125,    -20,      9, &
   22,     15,     11,      2,      2,     -4,    -12,     -7,    -10,     -8, &
   -5,    -13,    -13,    -18,     -6,    -19,     -7,    -18,      2,    -15, &
  -11,     -9,    -13,    -17,     -8,    -25,    -22,    -32,    -26,    -27, &
  -36,    -30,    -41,    -58,     -3,   1564,   1882,   1767,   1935,   1830, &
 2256,   2253,   1901,   1237,    865,    533,    387,    256,    228,    147, &
  268,    127,    334,    598,    624,    247,      3,    -20,    -27,    -26, &
  -32,    -20,    -16,      1,      8,     18,     24,     33,     35,     47, &
   46,     49,     60,     56,     51,     51,     57,     58,     61,     53, &
   44,   1183,    857,   1337,   1028 /)

  CASE(26)
  ellGrdHlp(1:145) = (/ &
  766,    172,      6,      5,    241,     75,    378,    175,    861,    810, &
  -18,    489,   1223,   1378,   1332,   1866,   2544,   2282,   2450,    273, &
  -27,     31,    255,     85,    135,    196,    424,   1308,   2398,   3380, &
 4001,   2191,   1383,   1054,    816,    826,    810,   1270,   2044,   2863, &
 1427,   1371,   1317,   1455,   1254,   1480,   1448,    296,    119,    312, &
  427,   1180,     -2,     -7,     22,     26,    172,    121,     29,     24, &
   23,     17,     12,      2,      1,     -6,    -16,    -11,    -11,    -10, &
   -5,    -15,    -10,    -13,     -3,    -15,     -5,    -22,     -5,    -17, &
  -18,    -12,    -16,    -23,    -17,    -32,    -28,    -35,    -21,    -15, &
  -30,    -29,    -41,    -62,    -35,    935,   1989,   1918,   2315,   1922, &
 2327,   2775,   2279,   1272,    747,    461,    299,    222,    180,    183, &
  269,    289,    438,    292,     72,    -30,    -21,    -32,    -32,    -24, &
  -35,    -30,    -26,     -9,      3,     14,     15,     25,     27,     35, &
   41,     50,     64,     62,     59,     59,     59,     55,     50,     46, &
   52,    727,   1177,   1022,    766 /)

  CASE(27)
  ellGrdHlp(1:145) = (/ &
   70,     27,     -8,    -27,      7,    193,    638,      4,     35,   1207, &
  150,    575,   1352,   1254,   1837,   1802,   1079,   1893,   1918,   2313, &
 -237,     73,     94,   1556,    160,    231,    303,   1597,   2252,   5093, &
 4964,   1766,   1304,   1111,   1356,   4558,   4214,   2858,   3391,   4961, &
 4502,   2764,   1806,   1644,   1335,   1572,    125,    -36,     30,     34, &
   60,    642,    -24,     -6,     47,     -3,    987,     15,     29,     10, &
    6,      5,     10,      1,      3,     -2,    -14,    -11,    -10,    -18, &
  -11,    -16,    -18,    -17,     -8,    -15,     -6,    -17,     -8,    -13, &
  -13,    -13,    -14,    -22,    -18,    -30,    -26,    -36,    -28,    -34, &
  -49,    -41,    -30,    -22,      6,    285,   1900,   2070,   2116,   2524, &
 1884,   3346,   2496,   1275,    742,    416,    241,    286,    182,    101, &
  205,    351,    813,     56,    -37,    -44,    -36,    -43,    -32,    -22, &
  -26,    -13,    -18,     -7,     -0,     12,     14,     25,     24,     40, &
   44,     51,     60,     57,     63,     53,     46,     34,     38,     34, &
   46,    430,    553,   1247,     70 /)

  CASE(28)
  ellGrdHlp(1:145) = (/ &
  192,    856,   1113,   1178,    550,     52,     57,     21,     43,     74, &
  139,     75,    340,    450,     55,    526,    444,    309,   1440,   2133, &
 2275,   2542,   1778,   1399,   1618,    579,   1693,   2355,   4156,   4330, &
 5096,   5812,   5356,   5427,   5288,   5499,   5624,   5037,   4356,   3922, &
 3994,   2908,   2198,   1470,   1113,   1297,     40,    424,    102,     10, &
    5,    698,    131,     34,    127,   1644,    256,      8,     21,     13, &
   11,      9,     12,     -4,     -8,    -10,    -11,     -9,     -8,    -13, &
   -8,     -7,     -9,    -13,    -12,    -16,    -11,    -16,    -12,    -12, &
  -12,    -15,    -15,    -23,    -22,    -25,    -20,    -32,    -24,    -33, &
  -43,    -46,    -48,    -47,    -57,    -68,     89,   1808,   1388,   1911, &
 2028,   2112,   2345,   1258,    749,    318,    258,    327,     38,    177, &
  399,   1069,    239,     11,    -32,    -43,    -45,    -50,    -40,    -33, &
  -33,    -27,    -27,    -17,    -12,      2,     10,     23,     29,     43, &
   48,     48,     45,     44,     49,     46,     48,     45,     49,     52, &
   60,     20,    285,     72,    192 /)

  CASE(29)
  ellGrdHlp(1:145) = (/ &
 1208,   1252,    469,    103,    164,     80,     56,     20,     19,    -25, &
  -21,     -5,    -15,     -9,    214,    986,    553,    242,    244,   2187, &
 2458,   1456,   1271,   1542,   1517,   1909,   2819,   3638,   2698,    763, &
 3673,   5035,   5580,   5632,   5498,   5649,   5636,   5101,   5767,   5216, &
 4327,   3706,   2330,   2110,   1412,    831,     46,    103,     72,      1, &
   -7,     11,    101,    450,    388,    -55,     29,     21,     24,     29, &
   28,      5,     13,    -10,     -2,      5,     -1,     -2,    -11,    -19, &
   -9,     -4,     -8,    -10,    -16,    -14,    -14,     -6,    -16,    -12, &
  -14,    -20,    -16,    -17,    -18,    -15,    -16,    -39,    -41,    -36, &
  -40,    -42,    -49,    -35,    -27,     -8,    152,   1352,    556,   1109, &
 2181,   2330,   1717,   1181,    486,    242,    165,     61,     51,    167, &
  302,    100,     31,    -40,    -35,    -59,    -48,    -50,    -38,    -31, &
  -34,    -31,    -25,    -12,    -10,     -1,      6,     15,     20,     31, &
   36,     38,     43,     43,     44,     35,     37,     33,     36,     31, &
   38,    146,   1039,   1174,   1208 /)

  CASE(30)
  ellGrdHlp(1:145) = (/ &
  885,    689,    277,    213,    398,    548,    111,     33,    129,    274, &
  113,     21,     14,      2,    365,    862,    821,    344,     13,    166, &
 2458,   2170,   1893,   1089,   1963,    856,   1432,   2120,   1501,    304, &
  215,   4475,   5302,   5289,   5140,   5364,   5040,   5146,   5189,   5369, &
 4457,   3862,   1052,   1265,   1356,    184,    204,     88,    134,     36, &
   16,      1,    159,    -36,     26,     53,     67,     17,     11,     18, &
   19,      9,     18,      2,      0,     -5,    -16,    -18,     -7,     -1, &
   -8,    -10,    -11,     -8,    -15,     -6,    -11,     -5,    -15,     -9, &
  -11,    -12,    -14,    -18,    -28,    -20,    -20,    -32,    -35,    -30, &
  -28,    -23,    -26,    -26,    -26,    -30,    -67,    -21,    335,    582, &
 1594,   1419,   1550,    880,    620,    256,     97,     30,     88,     38, &
  108,     37,    -29,    -35,    -36,    -46,    -39,    -54,    -42,    -35, &
  -33,    -34,    -26,    -13,    -12,    -11,      0,     17,     29,     30, &
   30,     34,     37,     32,     26,     26,     44,     53,     48,     45, &
  103,    942,   2146,   1168,    885 /)

  CASE(31)
  ellGrdHlp(1:145) = (/ &
  541,    570,    302,    356,    501,    684,    321,    367,    166,    186, &
  172,    111,    222,    340,    793,    983,    834,    519,    271,     59, &
   19,   2524,   2238,   2488,   1594,    722,   1041,   2055,   1276,    149, &
  167,    378,   3767,   5612,   5764,   6045,   5594,   5233,   4960,   4913, &
 4742,   3093,    467,    791,   1424,     37,    221,    519,    472,     29, &
   26,     37,    114,     33,     75,     55,     45,      4,     33,     46, &
   20,     -5,     12,      2,      2,      4,     -4,     -7,     -7,    -11, &
  -19,    -10,      3,      8,    -13,      0,     -8,      0,    -11,     -7, &
  -18,    -13,    -14,     -8,    -25,    -15,    -19,    -25,    -26,    -22, &
  -34,    -42,    -50,    -59,    -61,    -34,    -14,     22,    671,    304, &
 1160,   1935,   1384,   1013,    707,    169,    -31,    -27,    -11,    -28, &
  -29,      9,    -26,    -32,    -50,    -62,    -42,    -58,    -49,    -43, &
  -41,    -47,    -38,    -24,    -11,     -6,      6,     10,     15,     20, &
   28,     31,     30,     31,     35,     32,     44,     37,     48,     41, &
  246,   1387,    757,    645,    541 /)

  CASE(32)
  ellGrdHlp(1:145) = (/ &
  323,    671,    574,    543,    635,    638,    576,    600,    255,    185, &
  182,    233,    203,    666,    377,   1122,    954,    727,    459,    270, &
  -46,    903,   1512,   1196,   1303,   1162,   1309,    926,     24,    127, &
  411,    167,    103,   1344,   3522,   5741,   5195,   4834,   1054,   3770, &
 4661,   2925,   1252,    908,    675,    307,    332,    587,    829,     -3, &
   28,     34,     50,     31,     42,     42,     48,     45,     39,     27, &
   10,     24,     30,     11,     -0,      5,     -1,     -1,     -5,     -2, &
   -4,     -4,     -9,     -5,     -9,      9,      2,     -2,     -7,      5, &
    4,     -3,    -11,     -8,    -24,    -14,    -17,    -27,    -36,    -28, &
  -43,    -52,    -43,    -40,    -39,    -45,    -60,    -82,    -74,    208, &
  530,   2469,   1454,   1485,    231,     32,    -40,    -12,    -27,    -43, &
  -15,      9,    -43,    -37,    -51,    -51,    -48,    -52,    -47,    -53, &
  -48,    -43,    -30,    -29,    -20,    -21,     -4,      3,      9,     14, &
   21,     21,     26,     30,     30,     23,     25,    166,     92,    122, &
  524,    592,    558,    500,    323 /)

  CASE(33)
  ellGrdHlp(1:145) = (/ &
  285,    322,    949,   1197,    928,    746,    575,    515,    300,    261, &
  440,    357,    341,    349,     41,    750,   1229,    943,    762,    499, &
   37,    -56,     18,    603,    776,    550,    763,    501,     73,    159, &
  442,    304,     79,      5,     93,     58,    209,    573,   2056,   1422, &
 2879,   2435,   1868,   1203,    975,    527,    527,    895,    104,    -20, &
   25,     44,     25,     47,     40,     23,     40,     36,     49,     42, &
   21,     23,     11,      8,     17,     16,     -6,      1,     -3,     -8, &
   -7,     -3,     -6,      2,     -4,     -5,     -5,     10,      9,     13, &
    6,     -5,     -7,      0,    -23,    -21,    -22,    -20,    -25,    -21, &
  -32,    -31,    -35,    -58,    -57,    -56,    -51,    -35,    -30,    278, &
  -11,   1667,   1920,   1390,    805,    -19,    -75,     -4,    -22,    -53, &
  -22,     -8,    -26,    -26,    -56,    -44,    -40,    -46,    -44,    -50, &
  -48,    -50,    -39,    -40,    -23,    -18,     -1,     -0,      4,      7, &
   10,     14,     18,     18,     16,     25,     34,     34,     61,    346, &
  579,    420,    413,    337,    285 /)

  CASE(34)
  ellGrdHlp(1:145) = (/ &
  303,    600,   1380,   1456,   1311,    827,    654,    536,    675,    538, &
  796,    578,    272,    315,    643,    145,    907,    998,    794,    623, &
  186,     31,    127,    257,    -85,    -57,    -63,    -35,     43,    273, &
  426,    484,    437,    473,    479,    167,    -14,    561,    472,   1178, &
 2107,   1934,   1557,    599,    443,    535,    504,    141,     89,   -130, &
   21,     35,     12,     47,     39,     31,     48,     34,     37,     41, &
   15,     28,     22,     15,     10,     12,      1,      9,      1,     -5, &
   -0,     -2,    -10,     -4,      6,      5,      9,      6,     -7,     -3, &
   11,     -6,      6,     -9,    -20,    -10,    -12,    -18,    -16,    -20, &
  -38,    -36,    -37,    -52,    -35,    -40,    -33,    -17,    -41,    -45, &
  408,    -23,   2545,   2085,   2167,     30,    -45,    -37,    -22,    -16, &
  -15,    -28,    -32,     -9,    -50,    -43,    -55,    -39,    -48,    -52, &
  -55,    -55,    -36,    -37,    -28,    -18,     -3,     -4,     -6,      2, &
    7,     10,     16,     26,     20,     22,     26,     33,    190,    317, &
  327,    339,    312,    290,    303 /)

  CASE(35)
  ellGrdHlp(1:145) = (/ &
  449,    550,    907,    941,    786,    928,    824,   1068,    910,    536, &
  912,    535,    332,    371,    880,     73,   1016,   1110,    827,    489, &
  200,     83,    124,    384,      2,    -46,    -39,    -53,     55,     -7, &
  412,    518,    513,    312,    434,      2,    -37,    489,    163,   1354, &
 1684,   1271,    599,    350,    224,    245,     -5,     12,     81,     56, &
   72,     28,     21,     52,     24,     39,     60,     39,     33,     44, &
   18,     32,     28,     19,     18,     16,      1,      4,      7,      9, &
    6,      1,      8,     -1,     -0,      6,      8,      8,     17,      4, &
   23,    -11,     31,      9,      1,    -13,    -15,    -20,    -29,    -30, &
  -33,    -33,    -29,    -43,    -39,    -48,    -45,    -44,    -60,    -59, &
  -65,     10,    857,   2199,   1648,     21,    -26,     -9,      8,    -22, &
  -23,     25,    161,      9,    -61,    -50,    -78,    -38,    -47,    -52, &
  -47,    -43,    -36,    -37,    -31,    -25,    -15,     -9,     -8,     -2, &
   10,     17,     15,     22,     21,     28,     25,     19,    317,    390, &
  464,    425,    414,    312,    449 /)

  CASE(36)
  ellGrdHlp(1:145) = (/ &
  492,    791,    576,    688,    587,    528,    522,   1409,    595,    643, &
  770,    551,    384,    360,    671,     76,     38,   1677,    862,    477, &
  276,     91,    165,     73,    -45,    -42,    -48,    -56,    -57,    -54, &
  517,    410,    230,    503,    265,    -53,    -13,    -58,    225,   1163, &
  840,   1218,    677,    -36,    136,     -6,     -4,     26,     28,     66, &
   30,     44,     46,     41,     39,     60,     47,     46,     51,     41, &
   38,     38,     26,     26,     25,     16,     15,     14,      9,      7, &
   15,      8,     11,     -1,      4,      5,      5,      9,     12,     -9, &
   32,    -39,    452,    -22,     -7,    -13,    -27,    -26,    -33,    -27, &
  -16,    -34,    -33,    -41,    -53,    -62,    -55,    -59,    -46,    -23, &
  -37,    -18,   1041,   2090,   2738,   1933,     12,    -54,     38,      5, &
  -27,    -30,      1,     43,    166,     94,    -50,    -69,    -53,    -35, &
  -45,    -49,    -44,    -38,    -39,    -23,    -13,    -16,    -13,     -3, &
   -1,     10,     19,     27,     20,     24,     26,     27,    153,    391, &
  487,    417,    425,    392,    492 /)

  CASE(37)
  ellGrdHlp(1:145) = (/ &
  441,    512,    492,    598,    746,    510,    441,    412,    433,    741, &
  649,    549,    375,    430,    548,    818,     38,   1786,   1136,    695, &
  466,    441,    416,    -50,    -40,    -48,    -55,    -75,    -70,    -76, &
  513,    496,    191,    704,    -32,    -70,    -67,    -52,    306,    661, &
  498,    364,    757,    -19,     62,    -16,     30,    -10,    109,    180, &
   42,     27,     32,     47,     49,     51,     54,     58,     47,     34, &
   47,     32,     38,     31,     20,     23,     18,     12,     20,     14, &
    7,      4,      7,      5,     12,     11,     11,      6,     11,     -0, &
   10,     46,    -60,    -37,     -4,    -13,    -19,    -16,    -23,    -25, &
  -31,    -40,    -43,    -48,    -42,    -51,    -53,    -33,    -36,    -43, &
  -24,    -23,    -31,    684,   1790,   2338,    151,    169,    136,    -35, &
  -21,    -15,      7,    246,     -1,    418,      9,     -2,    -22,    -24, &
  -62,    -55,    -46,    -36,    -35,    -27,    -23,    -15,    -18,    -10, &
   -2,      6,     14,     21,     37,     25,     22,     27,     88,    249, &
  330,    390,    416,    397,    441 /)

  CASE(38)
  ellGrdHlp(1:145) = (/ &
  357,    509,    420,    468,    509,    436,    386,    278,    470,    976, &
  847,    753,    593,    467,    519,   1009,   -137,    182,   1704,    964, &
 1115,    184,    -78,    -37,    -22,    -77,    -61,    -89,    -74,    -77, &
  614,    343,     95,    -68,    -90,    -90,    -61,    -78,    -31,    -42, &
  378,    194,    137,   1011,      2,      6,     16,     13,    204,    120, &
   43,     37,     35,     47,     47,     49,     42,     60,     66,     33, &
   45,     35,     43,     28,     30,     16,     17,     13,     16,     12, &
   12,      7,      4,      6,     14,      9,     15,     14,     13,     14, &
   16,     26,      2,    -22,    -14,    -13,     -0,    -12,    -16,    -26, &
  -39,    -40,    -41,    -38,    -39,    -38,    -40,    -47,    -38,    -23, &
  -27,    -62,    -31,     91,    -89,   1113,    254,   1692,    453,    143, &
  141,    -13,     10,      1,    -34,     30,    -81,    -31,    -50,    -25, &
  -74,    -47,    -43,    -37,    -47,    -25,    -30,    -23,    -19,     -8, &
   -1,     -4,      9,     18,     12,     36,     23,     27,     89,    186, &
  298,    320,    343,    333,    357 /)

  CASE(39)
  ellGrdHlp(1:145) = (/ &
  377,    304,    356,    438,    507,    347,    350,    332,    510,   1055, &
 1241,    832,    690,    402,    592,   1472,   1498,    -43,   2276,    667, &
  -62,    -25,    -53,    -33,    -46,    -64,    -65,    -76,    -87,    -67, &
  461,    664,     59,    -94,    -90,    -95,    -73,    -73,    -46,      2, &
   63,    224,    177,    615,    -16,     25,     19,     39,     59,    155, &
   67,     44,     58,     49,     40,     50,     42,     55,     80,     28, &
   45,     41,     29,     31,     32,     21,     23,     21,      9,     16, &
   11,     14,     18,      5,      9,     12,     12,     12,     18,     15, &
    4,     11,      4,     -8,      1,      0,    -10,    -12,    -10,    -24, &
  -24,    -28,    -35,    -29,    -38,    -42,    -41,    -47,    -38,    -40, &
  -38,    -22,    -12,     12,    -14,     16,     -3,    -59,    950,   1237, &
  810,     13,      8,    -21,    -57,    212,    -47,    -43,    -26,    -52, &
  -42,    -43,    -53,    -39,    -47,    -35,    -25,    -22,    -16,    -15, &
   -7,     -5,     12,      9,     20,     19,     21,     34,     85,    151, &
  352,    426,    314,    363,    377 /)

  CASE(40)
  ellGrdHlp(1:145) = (/ &
  359,    298,    301,    662,    510,    423,    299,    434,    633,    899, &
  842,    697,    765,    509,    545,   2411,   1579,    654,     31,    -57, &
    1,    -17,    -52,    -39,    -56,    -48,    -79,    -79,    -79,   -105, &
  -61,    692,    -63,    -96,   -105,    -87,    -58,    -70,    -39,    -14, &
   99,    145,     68,    748,     -4,     15,     25,     51,     51,    139, &
  301,     46,     57,     41,     58,     61,     42,     43,     43,     33, &
   44,     38,     36,     34,     33,     32,     27,     24,     23,     17, &
   15,     16,      3,     12,     18,      6,     16,     13,      9,      8, &
    4,     19,      9,     -4,      0,      6,     -2,     -5,    -15,    -17, &
  -18,    -26,    -18,    -33,    -42,    -38,    -39,    -44,    -47,    -37, &
  -24,    -23,    -21,    -10,     -1,      8,    -19,    -58,     -2,     -1, &
  502,    -20,     16,     -1,   -120,      7,     14,    -22,      9,    -13, &
  -65,    -42,    -47,    -46,    -48,    -38,    -17,    -25,    -19,    -19, &
   -1,     -8,      8,     16,     13,     18,     15,     21,     72,    680, &
  561,    470,    433,    369,    359 /)

  CASE(41)
  ellGrdHlp(1:145) = (/ &
  276,    419,    322,    692,    698,    643,    473,    443,    468,    572, &
  662,    504,    453,    462,   1814,   2480,   2153,   1523,   1056,   1235, &
  636,    -32,    -57,    -44,    -71,    -56,    -80,    -76,    -92,   -101, &
  -75,    722,    -94,   -102,    -94,    -68,    -53,    -62,    -50,    -38, &
  -29,    -20,     46,      2,     43,     40,     13,     54,     65,    386, &
  209,     36,     50,     52,     72,     77,     74,     43,     55,     46, &
   41,     49,     38,     48,     38,     33,     26,     32,     26,     23, &
   19,     14,     16,      9,     10,     12,     13,      6,     11,     11, &
    9,     12,      1,      6,     -1,      6,     -4,      1,     -9,    -15, &
  -19,    -24,    -27,    -37,    -36,    -42,    -38,    -42,    -37,    -29, &
  -34,    -28,    -14,    -35,     -5,     -7,    -27,      7,      1,      5, &
  415,    180,      3,     -5,    110,   1003,   1303,    991,    240,     62, &
  -43,    -50,    -45,    -49,    -35,    -41,    -26,    -30,    -10,    -12, &
   -4,     -6,      8,     10,     12,     21,     12,     25,     17,    558, &
  529,    509,    397,    375,    276 /)

  CASE(42)
  ellGrdHlp(1:145) = (/ &
  264,    251,    518,    310,    235,    955,    792,    445,    540,    843, &
  893,    536,    410,    426,   1629,   2378,   2313,   1035,    905,    736, &
   83,    -43,    -42,    -58,    -70,    -53,    -89,    -96,    -81,   -115, &
  -90,    -25,     -3,    -97,    -75,    -69,    -59,    -50,    -46,    -22, &
  195,    -30,     -6,     21,     29,     15,     41,     71,     70,    234, &
 1268,     36,     61,     63,     85,     69,     72,     63,     50,     58, &
   56,     57,     45,     54,     45,     44,     33,     32,     29,     18, &
   22,     21,     10,     11,     13,      2,     10,     19,      6,     14, &
   15,      5,     -1,      8,      7,     11,      1,     -5,    -13,     -8, &
  -23,    -25,    -35,    -27,    -37,    -41,    -33,    -35,    -29,    -48, &
  -32,    -31,    -16,    -32,     -8,    -11,    -10,     20,     -0,     20, &
   85,    217,    112,    466,    354,   1825,    130,    101,    273,    407, &
  105,    -50,    -22,    -50,    -36,    -45,    -27,    -27,    -16,    -10, &
   -1,     -2,      2,      7,      9,     18,     17,     21,     27,    163, &
  565,    601,    339,    365,    264 /)

  CASE(43)
  ellGrdHlp(1:145) = (/ &
  244,     35,     78,    235,   1402,    920,   1027,    654,    574,    713, &
  798,    712,    524,    401,   1315,   2410,   1835,    599,    612,    214, &
  -37,    -65,    -53,    -53,    -55,    -75,    -86,    -93,    -95,   -104, &
  -90,   -114,    -71,    -99,    -81,    -76,    -62,    -50,     -9,    -68, &
   26,    205,     18,    -15,      9,     21,      7,    461,     79,     87, &
  649,     60,     77,     68,     74,     43,     57,     77,     50,     74, &
   63,     50,     49,     63,     50,     47,     36,     31,     26,     26, &
   24,     15,     21,     17,      5,     12,      9,     12,     17,     16, &
   16,     12,     10,     10,      6,     13,      5,     -1,    -12,    -14, &
  -22,    -20,    -28,    -25,    -30,    -28,    -34,    -35,    -32,    -31, &
  -23,    -39,    -13,    -30,      1,     -8,     -7,     12,     17,      4, &
   91,    -18,      9,    215,   1742,   3112,    151,    279,   1253,   1110, &
  553,     66,     39,    -41,    -41,    -37,    -26,    -26,    -14,     -6, &
   -9,      4,      1,      7,     12,     11,     22,     18,     22,     32, &
   92,    299,    204,    249,    244 /)

  CASE(44)
  ellGrdHlp(1:145) = (/ &
   29,     12,     30,      5,    330,    731,    660,    582,    588,    565, &
  724,    777,    997,   1310,    821,   1235,   1020,    260,    305,     35, &
  -36,    -66,    -53,    -56,    -61,    -62,    -80,    -94,    -94,    -99, &
 -100,   -105,   -123,   -108,    -94,    -77,    -55,    -58,    -63,   1444, &
  -10,    571,     16,     12,     22,     -3,    992,    275,     82,     37, &
  101,     70,     58,     69,     79,     39,     48,     84,     55,     57, &
   66,     53,     48,     60,     48,     40,     33,     36,     32,     23, &
   19,     22,     12,     16,     21,     15,     11,     14,     11,     14, &
   23,     17,     16,     16,      4,     10,      2,      2,     -4,    -12, &
  -14,    -19,    -17,    -19,    -21,    -28,    -32,    -32,    -29,    -20, &
  -25,    -23,    -18,    -21,      6,    -20,     10,    -14,      8,     -4, &
   27,     16,     21,     42,   2146,     57,    207,    354,   1113,    744, &
  422,    230,    231,    168,    -29,    -28,    -32,    -21,    -17,     -9, &
  -10,      1,      3,      8,     14,     18,     17,     13,     20,     25, &
   24,     37,     24,     13,     29 /)

  CASE(45)
  ellGrdHlp(1:145) = (/ &
   17,     13,     10,      5,    481,    683,    598,    407,    333,    389, &
  518,    713,   1417,   1134,   1764,   1133,    383,    194,     49,    -50, &
  -27,    -59,    -57,    -59,    -50,    -73,    -76,    -71,   -103,    -96, &
  -95,   -104,   -120,   -106,    -94,    -86,    -64,    -61,   -113,    -62, &
  360,     46,      8,     79,    105,    517,   1301,    464,     21,    -50, &
  186,    191,     68,     25,     66,     74,     42,     92,     47,     41, &
   62,     45,     43,     48,     42,     41,     39,     41,     25,     28, &
   30,     16,     21,     21,     15,     22,     13,      9,     14,     17, &
   14,     33,     22,      5,     12,      9,      3,     -1,     -0,     -5, &
   -9,    -16,     -7,    -19,    -15,    -30,    -21,    -31,    -23,    -23, &
  -23,    -20,    -22,     -9,    -16,     -6,    -20,    -21,    -20,    -14, &
   28,      9,     38,   1698,   1054,    480,    256,    234,    845,    384, &
  319,    212,    374,    290,    -43,    -22,    -26,    -19,    -18,    -11, &
   -9,      5,      5,     13,     12,     10,     13,      6,     15,     24, &
   21,     18,     21,     12,     17 /)

  CASE(46)
  ellGrdHlp(1:145) = (/ &
   15,     12,      8,     19,    187,    409,    497,    358,    383,    379, &
  450,    886,   1798,   1191,   1976,   1994,    205,     -7,    -61,    -38, &
  -48,    -50,    -48,    -56,    -61,    -73,    -73,    -80,    -89,    -96, &
  -98,   -101,   -103,    -93,    -96,    -79,    -63,    -61,    -60,      9, &
  794,     71,     41,     42,    211,    635,    705,     87,    250,    224, &
   44,    211,    152,    196,     75,    132,     68,     65,     64,     53, &
   58,     46,     43,     42,     48,     43,     39,     36,     33,     24, &
   23,     28,     16,     21,     15,     17,     10,     21,     12,     17, &
   16,     14,     18,     13,     15,     11,     11,      0,      1,     -6, &
    1,     -9,     -6,    -24,    -16,    -23,    -14,    -29,    -20,    -25, &
  -20,    -17,    -23,    -16,    -19,     -9,    -24,    -17,    -25,     12, &
  -33,     31,    224,   2249,    281,    283,    222,    119,    169,     56, &
  152,    149,    253,    143,    -26,    -29,    -19,    -22,    -20,    -10, &
   -8,      2,      5,      6,      9,     13,     11,     17,     19,     22, &
   13,     19,     20,     16,     15 /)

  CASE(47)
  ellGrdHlp(1:145) = (/ &
   18,     14,     12,     18,    120,    757,    685,    344,    337,    440, &
  560,   1031,   1958,   1161,   1709,   1313,     39,    -35,    -48,    -36, &
  -60,    -27,    -48,    -64,    -58,    -76,    -70,    -76,    -85,    -92, &
  -98,    -94,    -87,    -92,    -97,    -66,    -61,    -44,    -19,    -39, &
  -45,    329,    -14,     17,     81,    105,    164,     89,   1877,     46, &
  141,    129,     69,    126,     17,    405,    219,     13,     78,    108, &
  106,     43,     44,     39,     58,     31,     39,     44,     30,     29, &
   26,     28,     19,     23,     10,     16,     18,     13,     17,     19, &
   11,     14,     13,     10,     19,     11,     12,      2,     -3,     -4, &
    1,     -8,     -2,    -20,    -15,    -22,    -18,    -14,    -22,    -19, &
  -21,     -8,    -22,    -24,    -11,    -22,     -3,    -21,    -27,     17, &
  -28,     40,    165,    842,    194,    144,    158,    112,      1,     25, &
   77,     60,     58,     27,     -4,     16,    -15,    -15,     -8,     -4, &
   -2,     -5,     -9,      2,      0,     16,      7,     19,     16,     22, &
   21,     14,     15,     17,     18 /)

  CASE(48)
  ellGrdHlp(1:145) = (/ &
   15,     20,     15,      8,     -8,    287,    583,    439,    432,    532, &
  623,   1004,   1613,   1285,   1673,   1301,    -11,    -35,    -43,    -43, &
  -43,    -38,    -28,    -53,    -58,    -62,    -71,    -71,    -84,    -83, &
  -92,    -91,    -94,    -89,    -84,    -69,    -45,    -43,    -51,    -36, &
   14,    700,     34,     17,     15,     75,    115,    123,    572,    282, &
   50,    122,     91,    139,    720,   2887,   1570,    438,    122,    132, &
   81,    359,     68,     53,     47,     59,     39,     50,     34,     36, &
   26,     27,     26,     18,     26,     15,     18,     18,     14,     17, &
   19,     13,     13,     15,     12,     10,     13,      1,      1,     -0, &
  -10,    -10,    -10,     -9,    -14,    -17,    -24,    -17,    -13,    -17, &
  -14,    -17,    -14,    -20,    -13,    -11,    -11,    -10,     -5,     -7, &
    4,    -36,   1546,    440,    130,    103,    135,     60,     44,     34, &
   37,     59,    108,    103,    121,    186,     44,     66,    297,     -1, &
   -9,     -5,     -9,     -4,      0,     11,      5,     12,     16,     21, &
   20,     20,     14,     20,     15 /)

  CASE(49)
  ellGrdHlp(1:145) = (/ &
   18,     15,     14,     12,      1,    110,    819,    774,    697,    741, &
  724,    697,   1198,   1199,   1238,   1055,    -27,    -33,    -28,    -43, &
  -51,    -34,    -55,    -37,    -52,    -55,    -67,    -69,    -78,    -71, &
  -87,    -95,    -91,    -90,    -73,    -60,    -55,    -63,    -40,    -14, &
   -3,     19,    144,     58,     10,     76,     47,     58,    168,     69, &
   58,     38,     87,     27,    119,    197,    -48,   1878,   2414,    729, &
  591,     54,    499,     86,     58,     59,     46,     52,     43,     35, &
   34,     29,     29,     25,     22,     16,     20,     11,     17,     15, &
   18,     16,     12,     13,      6,      2,      2,      5,     -4,      0, &
   -5,     -4,    -18,     -8,    -15,    -13,    -20,    -18,    -12,    -17, &
   -5,    -15,    -16,    -12,      0,    -15,     -8,    -11,     -6,    -22, &
   10,    -36,    937,   2437,    186,    156,    128,     97,     75,     18, &
   41,    123,    148,    281,    250,    264,    230,    212,    450,    308, &
   -0,    -11,     -9,     -3,     -2,      5,      3,     13,     14,     19, &
   17,     21,     15,     25,     18 /)

  CASE(50)
  ellGrdHlp(1:145) = (/ &
   15,     13,      7,     10,     10,      6,   1015,    783,    995,    914, &
 1044,   1135,   1389,   1371,   1562,    486,    -43,    -31,    -31,    -37, &
  -41,    -35,    -52,    -33,    -44,    -51,    -56,    -65,    -67,    -76, &
  -74,    -87,    -93,    -79,    -71,    -75,    -70,    -52,    -42,    -22, &
  -30,      0,     -3,    151,    266,    693,    269,    210,    128,    155, &
  224,     96,     44,     59,     66,    126,     54,    103,     56,   1827, &
  107,     46,     77,    161,     82,     62,     56,     51,     43,     43, &
   32,     36,     35,     26,     24,     19,     18,     15,     19,      8, &
   16,      7,      9,     10,     -2,      4,      1,     -4,     -3,      2, &
   -7,     -1,    -12,    -10,    -16,    -10,    -16,     -9,    -15,    -12, &
   -6,    -14,     -4,    -15,     -2,    -15,     -2,    -32,     -1,    -16, &
    2,     39,    -56,   3962,    374,    232,    224,    105,     86,     66, &
  154,    283,    383,    323,    329,    303,    344,    288,    589,    686, &
   96,     -5,    -12,     -2,     -7,      2,      2,     11,     18,     14, &
   14,     19,     18,     22,     15 /)

  CASE(51)
  ellGrdHlp(1:145) = (/ &
   13,     12,      7,     13,     11,     32,   1145,   1159,   1212,   1027, &
 1071,   1447,   1480,   1448,   1421,    762,     13,    -33,    -37,    -28, &
  -22,    -33,    -40,    -36,    -38,    -49,    -43,    -59,    -62,    -65, &
  -69,    -75,    -78,    -80,    -85,    -70,    -55,    -54,    -60,    -39, &
  -35,     -8,      8,    -18,      1,     38,     26,    -31,    261,      8, &
  214,     39,     76,     59,     67,     77,     67,     65,     74,    291, &
  579,     74,     78,     78,    287,     89,     55,     63,     46,     47, &
   38,     39,     34,     30,     27,     23,     23,     16,     17,     14, &
    8,      8,      5,     -2,      4,      2,     -4,     -2,     -3,     -5, &
   -3,     -3,    -14,     -2,    -16,    -10,    -16,    -12,     -7,    -12, &
   -3,    -16,     -6,    -13,     -7,    -18,    -12,    -15,    -12,      1, &
    6,    -12,     98,   3998,   1404,    348,    256,    142,    246,    247, &
  325,    367,    437,    337,    206,    408,    587,    481,    552,    264, &
  -18,     -8,     -8,    -12,     -5,      3,      3,      7,     10,     13, &
   13,     15,     12,      9,     13 /)

  CASE(52)
  ellGrdHlp(1:145) = (/ &
    8,      5,     18,     21,     11,     -4,   1756,   1417,   1293,   1248, &
 1390,   1313,   1181,   1174,    739,    498,    336,     -8,     -7,     -8, &
   -1,    -28,    -22,    -43,    -36,    -45,    -38,    -43,    -51,    -61, &
  -63,    -62,    -72,    -71,    -72,    -61,    -57,    -66,    -60,    -39, &
  -37,    -18,     -8,    -21,    -12,     24,     25,     12,     32,     20, &
   47,     82,     65,    121,     67,     19,     66,    167,     84,     64, &
   57,     40,     74,     71,     93,     55,     51,     61,     61,     51, &
   42,     46,     41,     37,     27,     32,     22,     14,     15,      9, &
    8,      5,      6,     -1,      3,     -3,      4,     -7,     -6,     -5, &
  -15,     -3,    -11,    -11,    -13,     -8,    -11,    -15,     -9,    -12, &
    0,    -12,    -12,    -11,    -25,     -0,    -11,    -14,     27,    -14, &
   -6,     18,    -47,     68,   3763,   1800,    366,    317,    224,    410, &
  468,    356,    465,    324,    277,    523,    719,    865,    439,     22, &
  -14,    -13,    -12,     -8,     -1,     -6,      3,      3,      7,     11, &
    9,      7,      9,      5,      8 /)

  CASE(53)
  ellGrdHlp(1:145) = (/ &
   14,     13,      7,     15,     12,    525,   1654,   1519,   1269,   1075, &
 1172,   1160,   1304,   1096,    923,    679,    351,     -4,     -2,     29, &
  460,    -35,    -16,    -26,    -33,    -34,    -33,    -43,    -43,    -50, &
  -62,    -61,    -60,    -60,    -63,    -61,    -59,    -60,    -51,    -39, &
  -46,    -36,    -23,    -14,    -19,     -4,      3,     -0,     31,     22, &
   39,    103,    151,    354,    186,     39,     69,    252,     92,     49, &
   62,     72,     56,     66,     72,     55,     66,    110,     66,     54, &
   48,     50,     47,     50,     34,     31,     28,     19,     11,     10, &
   10,      3,     10,     -3,      9,      1,     -5,     -3,     -9,     -6, &
   -4,     -8,    -17,    -13,    -10,    -13,     -3,     -7,     -6,     -9, &
   -8,      3,    -10,     -8,    -10,     -4,      7,    -10,    -18,      3, &
   -6,    -27,     33,     36,   3992,   4712,   3996,    477,    274,    280, &
  586,    468,    506,    491,    550,    859,    742,    815,    590,     -4, &
  -12,    -14,     -5,    -12,     -5,     -1,     -2,      4,     10,      5, &
    9,      9,     12,      7,     14 /)

  CASE(54)
  ellGrdHlp(1:145) = (/ &
   16,     12,     13,     13,     16,    508,   1227,   1285,   1233,   1060, &
 1128,   1158,    839,    532,    823,    648,     95,    -18,     43,    355, &
  289,    -15,    -20,    -31,    -18,    -19,    -31,    -31,    -40,    -49, &
  -51,    -50,    -49,    -69,    -61,    -47,    -52,    -54,    -47,    -56, &
  -50,    -35,    -33,    -14,    -13,    -34,    -16,     -4,     24,      9, &
  376,    461,    156,    254,    199,     58,     66,    150,    652,     69, &
   45,     57,     56,     70,     55,     57,     57,    221,     61,     58, &
   53,     49,    141,     48,     39,     23,     24,     17,     14,     17, &
   16,      1,     10,      4,      2,     -0,     -1,     -2,    -12,     -3, &
   -6,    -15,     -6,    -16,     -5,    -11,     -8,     -1,     -1,      6, &
  -12,     -3,     -8,     -1,      7,      2,    -15,     -6,      5,    -18, &
   -0,     12,     15,    -35,     49,   3427,   4235,   2727,    107,    467, &
  328,    196,    547,    544,    801,    999,    533,    786,    359,     -8, &
   -8,     -8,    -10,    -11,     -7,     -7,      1,      5,      1,      8, &
   19,      6,     15,     15,     16 /)

  CASE(55)
  ellGrdHlp(1:145) = (/ &
   15,     16,     29,     16,     34,   1014,   1165,   1092,   1146,   1021, &
 1038,   1008,   1183,   1477,    256,     -6,    -17,    -12,    506,   1278, &
  -13,    -25,    -13,    -20,    -17,     -9,    -23,    -26,    -31,    -29, &
  -36,    -46,    -49,    -57,    -46,    -47,    -50,    -53,    -53,    -47, &
  -44,    -37,    -35,    -29,    -17,    -21,    -13,     -7,      9,    113, &
  278,    573,    425,    301,    267,    283,     74,    206,    857,     38, &
   58,     54,     56,     65,     56,     56,     54,     64,     73,     61, &
   61,    201,     55,     47,     47,     25,     25,     23,     14,     10, &
   14,      7,      5,     -1,     81,      2,     -0,     -3,     -7,    -10, &
  -12,    -12,    -11,    -12,      1,     -7,     -9,      3,     -7,      1, &
    4,     -5,     -6,     -4,     -9,     -3,     -1,     -9,     -9,      4, &
   18,    -14,     -6,     40,     -4,     -3,   3255,   4167,   2993,    326, &
  557,    156,    276,    781,    591,    821,    696,    811,    142,      0, &
  -14,     -7,     -9,     -6,     -5,     -5,     -3,      3,     11,      7, &
   14,     21,     16,     14,     15 /)

  CASE(56)
  ellGrdHlp(1:145) = (/ &
   22,     15,     27,     25,     33,     31,   1260,   1594,   1232,    966, &
  967,   1225,   1255,   1146,     15,    -20,    -24,    -16,    229,   1763, &
   13,    -18,    -27,     36,     -6,    -10,    -14,    -12,    -19,    -19, &
  -30,    -38,    -37,    -52,    -39,    -46,    -55,    -44,    -43,    -47, &
  -39,    -34,    -41,    -40,    -20,    -19,    -15,      4,     53,    240, &
  278,    357,    426,    354,    339,    301,    333,    275,    748,    446, &
   52,     57,     54,     64,     48,     53,    138,     76,     96,     70, &
   59,     68,     50,     48,     68,     32,     28,     23,     13,     12, &
    8,      2,      6,     -2,      4,     -5,    -12,     -4,     -7,    -13, &
   -4,    -18,    -16,     -8,     -9,     -1,      1,     -8,     -3,      4, &
   -4,      4,     -3,    -15,     -2,    -13,    -15,      7,      7,     -7, &
   -7,     13,     23,     20,     30,      2,   1601,   3717,   3605,    605, &
  251,    166,    446,    421,    452,    880,    899,    862,     72,    -12, &
   -8,     -4,     -2,      0,    -10,     -1,      3,     -1,      9,     16, &
   19,     16,     10,     17,     22 /)

  CASE(57)
  ellGrdHlp(1:145) = (/ &
   15,     21,     24,     21,     27,     43,    962,   1741,   1338,   1100, &
  984,   1034,    688,    282,     89,      1,     -3,    -23,    656,    871, &
    6,    -14,      7,     -6,     -7,     -5,     -1,     -8,    -21,    -18, &
  -20,    -23,    -28,    -40,    -41,    -38,    -41,    -42,    -48,    -48, &
  -43,    -40,    -29,    -32,    -35,    -24,     42,    575,    334,    323, &
  349,    420,    504,    627,    482,    270,    376,    304,    359,    366, &
  177,     46,     51,     55,     51,     66,    104,     77,     53,     61, &
   51,     60,     54,     46,     48,     31,     23,     24,     14,      7, &
    8,      5,      2,     -1,     -2,     -5,    -14,    -10,    -13,    -16, &
   -4,    -10,    -16,    -11,    -16,     -4,      1,     -7,      0,     -4, &
   -9,     -5,     -4,     -5,      3,     -5,     -1,    -10,    -23,     20, &
    5,      8,     33,      6,      2,     40,   1362,   4993,   3535,    384, &
  137,    203,    535,    335,    510,    777,   1322,    768,     -3,    -11, &
   -4,     -6,    -18,     -1,     -1,     -4,      2,      7,     12,      9, &
    8,     16,     14,     13,     15 /)

  CASE(58)
  ellGrdHlp(1:145) = (/ &
   18,     15,     21,     28,     16,     13,    513,   1437,   1239,   1104, &
 1133,   1199,   1262,    238,    118,     15,     18,     -1,    470,    478, &
  -17,      1,     -2,     -7,     -2,     -2,     -9,     -4,     -8,    -16, &
  -21,    -20,    -26,    -26,    -33,    -35,    -43,    -44,    -48,    -48, &
  -43,    -43,    -33,    -29,    -37,    -16,    237,    580,    568,    442, &
  438,    473,    612,    800,    448,    201,    161,    233,    334,    517, &
  394,     43,     47,     57,     52,     61,     57,     55,     55,     49, &
   52,     55,     49,     48,     24,     30,     24,     20,     10,     11, &
    8,      4,      2,      1,     -7,      2,    -10,     -9,    -13,    -15, &
  -11,     -8,    -12,    -10,    -11,     -8,     -9,     -8,    -11,    -11, &
    5,      0,     -7,     -2,      1,     -9,      9,      9,    -18,     15, &
   -1,     -5,     21,     -8,    -32,     79,   1691,   4567,   2700,    226, &
  130,    123,    407,    573,    834,    672,    127,    -13,     -6,    -16, &
   -7,     -2,     -4,     -5,     -7,      2,      4,     -4,      3,     15, &
   11,     13,     11,     14,     18 /)

  CASE(59)
  ellGrdHlp(1:145) = (/ &
   17,     31,     20,     23,     21,     20,    258,   1312,   1003,   1049, &
 1299,   1525,   1801,    113,      9,      9,      4,     21,     37,     10, &
  -19,      9,      3,     -7,      2,     -1,     -0,     -0,     -5,    -15, &
  -16,     -7,    -24,    -25,    -32,    -31,    -39,    -38,    -42,    -46, &
  -38,    -44,    -38,    -29,    -40,    -21,    164,    550,    584,    502, &
  509,    577,    821,    617,    277,     52,    139,    205,    353,    606, &
  448,    313,     51,     48,     44,     45,     46,     54,     55,     52, &
   50,     40,     47,     57,     27,     29,     29,     18,     12,     17, &
    7,      1,      4,      3,     -4,     -1,     -7,    -10,     -9,     -5, &
  -17,    -11,    -15,    -13,     -6,     -2,    -14,     -9,      1,    -12, &
  -11,      3,      2,     -8,     -4,     -0,      7,      3,      4,     15, &
  -17,    -14,      9,     -2,      9,     84,   1965,   4488,   1263,    271, &
  190,    142,    273,    889,    983,    -17,      8,      6,    -11,      4, &
    1,    -20,      0,      9,     -4,     -4,      3,      5,     12,     12, &
    5,     19,     15,      9,     17 /)

  CASE(60)
  ellGrdHlp(1:145) = (/ &
   16,     32,     25,     25,     31,     36,      9,   1008,    901,   1327, &
 1334,   1522,   1582,     82,      2,     20,     12,      7,     13,     19, &
   -1,      6,      6,      5,     14,     12,      8,      4,      7,      2, &
   -9,     -7,    -18,    -19,    -26,    -26,    -32,    -33,    -33,    -38, &
  -37,    -44,    -41,    -30,    -44,    -33,    237,    481,    561,    539, &
  460,    471,    552,    402,    260,      2,     79,    170,    220,    251, &
  340,    608,     12,     24,     40,     42,     40,     50,     52,     45, &
   48,     50,     51,     46,     25,     33,     28,     15,     11,     16, &
    5,      1,      2,     -1,     -4,     -2,      4,     -6,     -6,     -9, &
  -19,    -10,     -8,    -17,    -14,     -4,     -8,    -12,     -2,     -7, &
  -11,     -8,     -6,    -10,      3,      8,     -3,    -12,     -2,      3, &
   -6,     25,     43,      5,     -6,     91,   3214,   2753,    535,    109, &
   71,     73,    266,    617,   1221,    -19,     -1,     -2,    -14,     -6, &
    1,     -0,      4,     -8,     -2,     19,     16,      3,     10,     20, &
    9,     16,     23,     19,     16 /)

  CASE(61)
  ellGrdHlp(1:145) = (/ &
   17,     24,     28,     34,     20,     21,     51,    591,    979,   1196, &
 1480,   2057,   1413,     16,     33,     10,      6,      8,     17,     14, &
   10,     17,     20,     16,     22,     14,     15,      7,     11,      7, &
   -3,     -1,     -9,    -15,    -18,    -19,    -28,    -26,    -27,    -39, &
  -32,    -36,    -47,    -41,    -45,    -33,    135,    410,    457,    432, &
  314,    247,    200,    259,    216,    119,    129,    140,    143,    180, &
  531,    894,     37,     17,     28,     26,     32,     38,     48,     48, &
   44,     46,     46,     41,     34,     25,     20,     15,     13,     11, &
    3,      2,      3,     -2,     -2,     -6,     -3,     -5,     -8,     -4, &
  -15,    -17,    -12,     -8,     -7,    -15,    -19,    -20,    -13,     -8, &
   -3,     10,     -2,     -9,     -8,    -11,     -3,      7,     17,     12, &
   -2,     -9,      4,     15,     20,      8,   4734,   1581,    373,     82, &
   82,    107,    198,    216,    147,     34,    -20,    -13,     -3,     -5, &
  -12,     -9,     15,      8,      1,      9,     11,     11,     18,     25, &
   16,     13,     23,     31,     17 /)

  CASE(62)
  ellGrdHlp(1:145) = (/ &
   18,     21,     21,     26,     33,     31,     22,     -0,   1246,   1382, &
 1552,   1422,    101,     17,     12,     -0,     17,     26,     31,     26, &
   24,     19,     20,     21,     17,     13,     14,      9,     11,      8, &
    2,     -2,     -4,    -10,    -13,    -18,    -25,    -23,    -26,    -34, &
  -33,    -39,    -44,    -39,    -38,    -53,    -58,    343,    384,    333, &
  135,     89,     -6,     89,    247,    261,    261,    138,    230,    265, &
  638,    438,     18,     22,     43,     38,     38,     31,     35,     41, &
   35,     41,     39,     17,     30,     25,     21,     18,     11,      5, &
    4,      6,      3,     -4,     -4,     -3,     -5,    -13,    -12,     -7, &
   -9,    -15,    -14,    -11,    -11,    -16,    -11,     -2,      1,     -7, &
  -19,    -10,     -7,     -3,     -2,     -2,      2,     -2,    -13,    -13, &
    6,     13,      8,     -6,     -7,    -42,   4528,    833,   1384,    155, &
   99,    126,    171,     98,     18,      3,     -7,     -5,     -2,     -7, &
   -6,     -1,      7,      0,     -0,     13,     21,     16,     11,     13, &
   18,     15,     16,     20,     18 /)

  CASE(63)
  ellGrdHlp(1:145) = (/ &
   22,     20,     12,      7,     26,     35,     31,     12,    804,    691, &
  397,     -8,     32,     35,     38,     26,     38,     18,     22,     26, &
   40,     24,     27,     27,     17,     16,     21,     15,     13,      9, &
   10,      4,      8,     -3,     -4,    -16,    -20,    -19,    -28,    -35, &
  -32,    -35,    -38,    -38,    -27,    -40,      8,    328,    116,     96, &
  -24,    -35,    -18,     -2,     97,     -1,    112,     69,    133,    336, &
 1026,     36,     30,     30,     29,     18,     36,     33,     36,     53, &
   26,     47,     39,     19,     28,     24,     20,     17,     12,      1, &
    3,      5,      1,     -6,     -6,     -7,     -7,    -13,    -12,    -17, &
  -11,    -11,     -7,     -7,    -10,    -11,    -10,    -11,    -13,    -10, &
  -12,     -4,    -10,    -13,    -23,    -19,    -11,     -6,     -5,      4, &
   13,      3,      2,      1,     38,    131,   4260,    563,    481,    122, &
   52,     97,    176,      5,     -4,     -0,      7,    -11,    -17,    -14, &
   -4,      7,     11,      7,     10,     16,     20,     13,     18,     14, &
   18,     10,     11,     12,     22 /)

  CASE(64)
  ellGrdHlp(1:145) = (/ &
   24,     23,     21,     19,     27,     28,     29,     27,     36,     59, &
   42,     23,     31,     28,     23,     25,     42,     32,     36,     36, &
   40,     32,     31,     29,     19,     21,     24,     23,     24,     14, &
   12,      8,     12,      3,      3,     -7,    -12,    -18,    -25,    -26, &
  -29,    -35,    -33,    -40,    -38,    -48,    -37,    -45,    -39,    -35, &
  -32,    -37,    -29,    -21,    -29,     27,     86,    118,    129,    793, &
  549,     10,      7,      5,     10,      9,     27,     27,     12,     24, &
   90,     40,     32,     24,     23,     17,     12,     11,     12,      6, &
    2,      1,     -1,     -2,     -1,     -7,     -9,    -12,     -8,    -14, &
  -10,    -12,    -17,    -15,    -16,    -14,    -10,    -12,    -13,    -16, &
  -18,    -12,     -9,     -7,     -0,     10,      9,      4,     -9,    -11, &
   -4,      4,     28,     16,     16,    213,   2763,    807,    287,     86, &
   91,     58,     16,     12,    -18,    -14,     -3,     -9,     -7,     -7, &
   -2,      8,     10,     11,     12,     13,     18,     17,     24,     22, &
   20,     14,     16,     15,     24 /)

  CASE(65)
  ellGrdHlp(1:145) = (/ &
   17,     17,     21,     20,     27,     27,     28,     22,     24,     32, &
   30,     31,     39,     33,     24,     29,     47,     34,     39,     38, &
   32,     31,     33,     35,     26,     27,     24,     20,     21,     12, &
   12,     10,     12,     -0,      4,     -3,     -7,    -12,    -19,    -20, &
  -28,    -36,    -30,    -37,    -33,    -44,    -33,    -42,    -38,    -39, &
  -29,    -31,    -27,    -19,    -27,    -13,     -5,    273,    310,    351, &
   37,    -15,      2,     12,     17,      5,     26,     28,     21,     42, &
  237,    470,      7,     20,     18,     12,      8,      9,     14,      8, &
    6,      1,     -2,     -2,     -2,     -7,    -10,    -14,     -9,    -15, &
   -4,    -11,    -10,     -9,    -15,    -12,    -10,     -9,     -8,     -8, &
   -8,     -5,     -5,     -6,    -10,     -1,     -5,     -8,     -2,      7, &
    9,      6,     11,    -24,    -16,    594,   1395,    455,    250,    337, &
  229,     95,      8,      9,    -13,     -7,      3,     -4,      4,     -6, &
   -7,     -1,      0,     12,     16,     18,     22,     15,     24,     28, &
   24,     21,     25,     14,     17 /)

  CASE(66)
  ellGrdHlp(1:145) = (/ &
   19,     18,     23,     19,     22,     29,     28,     30,     24,     21, &
   26,     37,     34,     34,     28,     32,     38,     36,     49,     47, &
   34,     42,     34,     30,     29,     29,     22,     21,     26,     22, &
   19,     17,     10,      1,      4,      2,     -2,     -5,     -7,    -10, &
  -18,    -27,    -29,    -35,    -29,    -37,    -26,    -29,    -29,    -36, &
  -30,    -21,    -34,    -25,    -16,    -16,    -18,     -7,    -11,      7, &
   -3,     10,     13,     13,     15,     -2,     19,     27,     -4,     52, &
  249,     43,     18,     20,     16,     11,     12,      6,      8,      5, &
   -1,     -4,     -7,     -5,     -4,     -6,     -5,     -8,    -10,    -16, &
  -13,    -20,    -17,     -9,    -11,    -12,    -14,    -17,    -13,    -15, &
   -9,     -7,     -8,     -5,     -7,     -3,     -2,     -7,      2,      5, &
    0,     -0,      2,     -5,     11,    804,   1056,    609,    116,     38, &
   23,     10,     -7,     -4,     -6,     -6,     -4,     -1,     -3,     -4, &
   -6,     -5,      0,     15,     18,     20,     26,     19,     21,     21, &
   35,     22,     26,     16,     19 /)

  CASE(67)
  ellGrdHlp(1:145) = (/ &
   23,     18,     23,     20,     22,     30,     30,     32,     29,     27, &
   30,     39,     32,     33,     36,     40,     39,     37,     49,     46, &
   37,     47,     36,     31,     31,     34,     28,     22,     30,     28, &
   25,     23,     15,      6,      8,      5,     -5,     -4,     -7,    -11, &
  -15,    -22,    -30,    -33,    -27,    -38,    -31,    -31,    -27,    -38, &
  -33,    -22,    -37,    -28,    -17,    -19,    -20,     -8,    167,    607, &
  -22,     -8,     -5,    -13,     -4,     -7,     26,     12,    -65,   1506, &
  110,     -3,     23,     16,     13,      6,      5,     -3,     -3,     -4, &
   -6,     -9,     -9,     -6,     -5,     -9,     -7,     -9,    -10,    -13, &
  -12,    -16,    -14,     -5,     -6,     -5,     -5,    -12,     -9,    -12, &
   -6,     -9,    -14,    -10,    -10,     -9,     -6,     -7,     -4,     -2, &
   -1,     -2,     18,     29,     70,    726,   1243,   1025,    145,     26, &
   32,      9,     -3,      6,     -4,     -4,     -8,     -9,     -7,     -6, &
   -3,      1,      7,     15,     16,     19,     24,     20,     22,     22, &
   24,     23,     27,     16,     23 /)

  CASE(68)
  ellGrdHlp(1:145) = (/ &
   23,     18,     21,     23,     22,     30,     24,     30,     31,     31, &
   34,     37,     28,     37,     45,     48,     42,     39,     42,     47, &
   45,     55,     41,     36,     33,     33,     35,     24,     28,     31, &
   20,     29,     22,     21,     16,     12,      2,      6,     -2,     -7, &
   -8,     -5,    -17,    -20,    -16,    -29,    -29,    -32,    -23,    -29, &
  -32,    -17,    -30,    -25,    -23,    -25,    -29,    -35,    -16,     24, &
   -6,     -7,     -7,     -8,      3,     11,     62,    157,   1649,    -62, &
   24,      7,      6,      9,      4,     -2,      0,     -3,     -3,     -7, &
   -6,     -1,     -4,     -9,    -10,    -15,    -14,    -21,    -23,    -19, &
  -22,    -17,    -18,    -17,    -15,    -14,    -14,    -19,    -15,    -19, &
  -12,     -9,    -12,     -1,     -1,     -7,     -4,     -1,     -2,     -7, &
    3,      3,     16,     22,     17,   1124,   1088,    434,    144,     24, &
   27,      5,      5,      4,     -5,     -2,     -9,     -8,    -11,    -11, &
   -7,      1,      7,      8,     12,     16,     22,     27,     30,     25, &
   29,     27,     28,     23,     23 /)

  CASE(69)
  ellGrdHlp(1:145) = (/ &
   27,     21,     22,     27,     25,     29,     26,     32,     33,     38, &
   36,     36,     31,     38,     45,     50,     42,     47,     42,     46, &
   62,     71,     41,     39,     33,     31,     37,     31,     31,     36, &
   24,     27,     23,     23,     18,     15,      7,      9,      1,     -8, &
  -10,     -9,    -17,    -18,    -16,    -24,    -25,    -30,    -23,    -27, &
  -31,    -22,    -25,    -20,    -23,    -18,    -13,    -14,    -17,    -11, &
  -14,    -15,    -12,    -14,    -14,    -17,    -22,    721,    432,     -9, &
   24,      3,     -2,     -4,     -4,     -5,     -4,     -6,     -6,     -7, &
   -6,     -2,     -4,     -8,     -7,    -11,    -11,    -14,    -16,    -15, &
  -17,    -13,    -13,    -13,    -14,    -12,     -6,     -9,     -7,    -11, &
  -12,     -9,    -13,     -6,     -4,    -12,     -4,      1,      2,      0, &
   15,     13,     19,     21,    158,   1363,    687,    201,      5,      7, &
    1,     -3,      1,     -4,    -10,      0,     -2,     -3,     -5,     -5, &
   -3,      4,      7,      8,     16,     23,     26,     29,     32,     26, &
   27,     21,     22,     24,     27 /)

  CASE(70)
  ellGrdHlp(1:145) = (/ &
   27,     22,     19,     28,     26,     25,     25,     31,     32,     44, &
   36,     41,     40,     45,     50,     46,     54,     61,     40,     48, &
   49,     38,     41,     49,     38,     25,     33,     24,     13,     45, &
   28,     31,     22,     23,     11,     15,      7,     12,      3,     -1, &
   -8,     -4,    -14,    -17,    -20,    -21,    -20,    -31,    -24,    -18, &
  -25,    -24,    -19,    -11,    -32,    -16,     -9,     -9,    -22,    -22, &
  -18,    -25,    -14,    -13,    -13,     -6,     -2,     11,    -23,    -19, &
   18,    -11,     -2,     -4,     -5,     -7,    -12,     -7,     -3,     -3, &
   -7,     -6,     -7,    -11,     -6,    -11,    -17,    -17,    -17,    -15, &
  -19,    -12,    -14,    -13,    -13,    -11,     -3,    -13,     -8,     -5, &
  -12,      0,     -7,     -4,      0,      0,     11,     11,      6,     -9, &
    7,    -16,    -18,      3,    316,   1384,    888,    327,     20,     20, &
   17,     16,     17,      3,     -7,      9,     -1,     -2,      2,      2, &
    0,      7,      6,      9,     19,     23,     12,     16,     25,     24, &
   34,     27,     23,     24,     27 /)

  CASE(71)
  ellGrdHlp(1:145) = (/ &
   26,     29,     27,     26,     25,     26,     31,     33,     28,     36, &
   29,     36,     38,     37,     42,     41,     49,     56,     43,     48, &
   51,     45,     41,     42,     36,     30,     37,     14,    102,     50, &
   36,     31,     22,     24,     14,     20,     13,      9,      2,      5, &
   -7,    -10,     -9,    -10,    -17,    -16,    -15,    -21,    -24,    -19, &
  -23,    -31,    -27,    -15,    -27,    -14,    -17,    -13,    -17,    -20, &
  -14,    -19,    -14,    -17,    -19,    -14,    -13,    -14,    -17,     -9, &
   -8,    -20,     -5,    -13,    -20,    -18,    -21,    -17,    -16,    -16, &
  -18,    -20,    -18,    -16,    -12,    -12,    -11,    -13,    -12,    -16, &
  -20,    -18,    -22,    -19,    -14,    -13,     -6,     -9,     -6,     -2, &
  -13,     -5,     -9,    -12,     -7,     -4,     -5,     -8,      2,      1, &
   17,     12,     15,      8,    197,   1104,    512,     78,     26,     18, &
    9,     10,      5,     -0,     -2,      6,      0,      6,      6,      4, &
    5,      8,     10,     14,     14,     17,     19,     25,     28,     21, &
   25,     26,     27,     26,     26 /)

  CASE(72)
  ellGrdHlp(1:145) = (/ &
   16,     26,     26,     28,     28,     31,     36,     37,     27,     36, &
   24,     30,     42,     41,     43,     38,     40,     51,     40,     45, &
   41,     41,     46,     44,     39,     33,     40,     28,     28,     35, &
   35,     33,     25,     29,     14,     20,      8,      7,      2,     12, &
   -1,     -9,     -8,     -9,    -19,    -18,    -16,    -16,    -27,    -23, &
  -19,    -29,    -25,    -17,    -34,    -14,    -26,    -18,    -15,    -22, &
  -14,    -21,    -17,    -16,    -14,    -18,    -23,    -20,     -9,    -18, &
  -28,    -27,    -12,    -31,    -22,    -19,    -28,    -24,    -26,    -25, &
  -20,    -22,    -21,    -23,    -20,    -16,    -13,    -11,    -11,    -18, &
  -22,    -17,    -16,    -13,    -10,    -17,    -11,    -12,    -11,      1, &
  -14,     -7,     -9,    -13,    -14,     -9,    -13,    -18,      3,     -3, &
    9,      6,     14,     -3,     62,    325,    259,     10,     25,     15, &
  119,     46,      7,      9,      4,     -0,     -2,      8,      6,     12, &
   14,     16,     16,     18,     14,     18,     19,     19,     26,     25, &
   28,     29,     32,     26,     16 /)

  CASE(73)
  ellGrdHlp(1:145) = (/ &
   25,     30,     25,     24,     27,     28,     30,     34,     27,     36, &
   32,     33,     39,     38,     42,     44,     42,     41,     43,     51, &
   42,     45,     44,     37,     43,     38,     32,     31,     31,     34, &
   37,     35,     24,     28,     17,     20,      8,      9,     -0,      5, &
    0,      0,     -3,     -8,    -13,    -14,    -20,    -15,    -24,    -26, &
  -21,    -26,    -27,    -23,    -28,    -14,    -34,    -26,    -20,    -23, &
  -13,    -20,    -22,    -23,    -19,    -21,    -23,    -20,    -22,    -24, &
  -28,    -30,    -32,    -40,    -25,    -31,    -34,    -27,    -30,    -29, &
  -27,    -31,    -24,    -21,    -19,    -21,    -26,    -22,    -17,    -18, &
  -16,    -18,    -17,    -16,    -12,    -17,    -14,    -12,    -19,     -8, &
  -15,    -14,    -13,    -13,    -14,     -9,     -8,    -17,     -4,    -10, &
   -3,     -2,     15,     -4,      9,    131,    584,    292,     33,      4, &
   17,     14,     -1,     17,     14,     11,     11,     12,     12,    148, &
   38,     19,     15,     15,     11,     21,     23,     18,     25,     28, &
   26,     22,     26,     29,     25 /)

  CASE(74)
  ellGrdHlp(1:145) = (/ &
   21,     27,     20,     22,     20,     22,     22,     32,     26,     31, &
   33,     31,     40,     39,     38,     41,     37,     40,     44,     47, &
   40,     44,     44,     31,     40,     37,     34,     32,     33,     35, &
   32,     34,     27,     25,     15,     20,      7,      7,      3,      5, &
   -2,     -2,      2,     -6,    -10,    -13,    -23,    -13,    -25,    -30, &
  -23,    -26,    -26,    -25,    -28,    -19,    -40,    -25,    -21,    -23, &
  -18,    -24,    -23,    -27,    -29,    -27,    -30,    -30,    -33,    -33, &
  -35,    -41,    -37,    -33,    -39,    -41,    -28,    -34,    -34,    -34, &
  -29,    -31,    -25,    -24,    -21,    -27,    -28,    -26,    -29,    -24, &
  -12,    -23,    -21,    -20,    -12,    -16,    -17,    -15,    -22,    -17, &
  -19,    -14,    -13,    -15,    -17,    -10,     -6,    -16,     -2,    -11, &
   -8,     -7,     14,     -5,      1,     -3,    -14,     25,      2,      7, &
   12,     16,     13,     27,     18,     15,     15,     16,     30,      8, &
   12,     16,     26,     24,      2,     16,     26,     21,     22,     19, &
   25,     26,     23,     22,     21 /)

  CASE(75)
  ellGrdHlp(1:145) = (/ &
   17,     22,     22,     24,     15,     22,     20,     26,     31,     28, &
   33,     38,     36,     27,     30,     48,     45,     39,     42,     37, &
   37,     37,     31,     26,     39,     32,     30,     36,     31,     35, &
   31,     29,     25,     26,     16,     12,     14,     16,      3,      3, &
    1,     -6,     -3,    -10,     -9,    -11,    -33,    -19,    -17,    -29, &
  -28,    -21,    -25,    -31,    -24,    -22,    -31,    -16,    -24,    -28, &
  -29,    -29,    -27,    -39,    -33,    -33,    -42,    -36,    -42,    -47, &
  -39,    -44,    -41,    -38,    -53,    -41,    -33,    -41,    -30,    -37, &
  -31,    -35,    -23,    -26,    -29,    -25,    -27,    -32,    -29,    -31, &
  -21,    -27,    -18,    -24,    -25,    -19,    -14,    -16,    -21,    -20, &
  -19,    -16,    -20,    -14,    -10,    -11,     -3,     -8,      3,     -9, &
   -6,     -8,      1,     -9,      9,     11,     -4,     10,     13,     12, &
   15,     20,     17,     23,     16,     21,     26,     16,     31,     20, &
   23,     23,     23,     22,     25,     10,     26,     19,     19,     20, &
   18,     18,     22,     25,     17 /)

  CASE(76)
  ellGrdHlp(1:145) = (/ &
   15,     21,     16,     21,     26,     19,     24,     27,     25,     28, &
   31,     32,     26,     25,     28,     40,     42,     33,     38,     32, &
   24,     31,     34,     16,     41,     32,     24,     36,     34,     28, &
   24,     26,     25,     17,     21,      8,     17,     10,      9,     -1, &
   -1,    -11,     -7,    -16,    -11,    -13,    -32,    -22,    -15,    -31, &
  -25,    -27,    -26,    -37,    -31,    -28,    -30,    -22,    -26,    -34, &
  -33,    -28,    -31,    -36,    -39,    -37,    -43,    -40,    -49,    -50, &
  -46,    -49,    -43,    -41,    -53,    -43,    -43,    -39,    -38,    -43, &
  -24,    -45,    -31,    -29,    -34,    -30,    -28,    -32,    -29,    -34, &
  -24,    -26,    -31,    -31,    -20,    -31,    -16,    -20,    -16,    -26, &
  -18,    -20,    -21,    -17,    -14,    -15,     -7,    -12,     -4,     -4, &
  -11,     -7,      6,    -11,     -3,     21,      0,     24,     13,     16, &
   29,     24,     18,     22,     23,     12,     44,     14,     25,     19, &
   32,     21,     24,     27,     14,     13,     29,     14,     20,     21, &
   13,     17,     14,     20,     15 /)

  CASE(77)
  ellGrdHlp(1:145) = (/ &
    8,     11,     23,     12,     25,     21,     25,     25,     28,     23, &
   10,     26,     17,     16,     23,     31,     25,     24,     30,     23, &
   14,     30,     31,     18,     47,     28,     21,     29,     21,     20, &
   24,     10,     15,     11,     24,      4,     16,      4,      9,    -12, &
   -5,    -21,    -13,    -38,    -18,    -10,    -29,    -37,     -1,    -38, &
  -25,    -39,    -31,    -49,    -48,    -41,    -44,    -25,    -31,    -49, &
  -31,    -38,    -27,    -35,    -40,    -37,    -45,    -33,    -50,    -56, &
  -51,    -50,    -49,    -40,    -52,    -44,    -48,    -43,    -43,    -38, &
  -41,    -46,    -37,    -46,    -36,    -34,    -29,    -25,    -32,    -26, &
  -31,    -33,    -26,    -38,    -27,    -35,    -24,    -30,    -15,    -29, &
  -14,    -24,    -24,    -20,    -20,    -24,    -19,    -16,    -14,    -15, &
   -3,     -0,     -6,      4,    -11,     30,      1,     42,      1,     43, &
   75,     46,     35,     11,     22,     11,     26,     13,     21,     15, &
   28,     19,     17,     19,     23,     10,     16,     11,     15,     11, &
   18,     13,      9,     24,      8 /)

  CASE(78)
  ellGrdHlp(1:145) = (/ &
   15,      7,     17,     18,      8,     24,     15,     21,     24,     10, &
   16,     15,     23,     24,     23,     26,     12,     27,     23,     40, &
   12,     48,     46,     21,     49,     25,     31,     22,     22,     15, &
   25,     17,     14,     23,     13,      9,     -2,     -1,     -1,    -16, &
    5,    -16,      8,    -47,    -13,    -10,    -10,    -40,    -14,    -46, &
  -32,    -52,    -37,    -43,    -51,    -41,    -64,    -42,    -48,    -47, &
  -40,    -45,    -31,    -41,    -32,    -49,    -44,    -38,    -47,    -48, &
  -50,    -55,    -50,    -46,    -54,    -39,    -57,    -47,    -46,    -51, &
  -41,    -50,    -49,    -36,    -60,    -31,    -43,    -38,    -33,    -38, &
  -42,    -33,    -38,    -31,    -35,    -33,    -33,    -32,    -34,    -26, &
  -30,    -20,    -31,    -24,    -18,    -33,    -18,    -24,    -22,    -12, &
  -11,     -2,     -6,      6,    -11,     28,    -22,     49,    -10,    290, &
  597,    515,     -6,     35,      6,     15,     20,     12,     12,     18, &
    9,     21,      8,     15,     12,     10,      7,     11,     11,      8, &
   15,      5,     17,      7,     15 /)

  CASE(79)
  ellGrdHlp(1:145) = (/ &
    1,     10,     11,     12,      5,     14,     12,     19,      5,      6, &
   10,     11,     39,     35,     45,      6,      8,     16,     64,     61, &
   30,    615,    677,    -11,     76,     32,     31,     63,     -1,     38, &
   10,     37,     30,     53,      3,    -28,    -50,     21,    -21,     47, &
   87,    610,    357,     21,     -3,    712,    102,    -38,    -56,    -87, &
  -33,    -76,     21,    223,    123,     88,   -135,    -88,    -77,    -43, &
  -48,    -57,    -43,    -25,    -39,    -34,    -65,    -36,    -36,    -53, &
  -39,    -72,    -50,    -59,    -49,    -46,    -57,    -61,    -54,    -56, &
  -44,    -64,    -35,    -57,    -53,    -51,    -53,    -48,    -47,    -39, &
  -67,    -31,    -45,    -28,    -37,    -33,    -39,    -44,    -46,    -39, &
  -32,    -22,    -47,    -12,    -33,    -26,    -28,    -24,    -37,    -15, &
  -14,    -16,     -5,     -3,      2,     -4,      0,    -47,    868,    730, &
 -110,    127,    -24,     40,     -6,     25,     10,     15,      8,     11, &
   -0,     14,      5,      7,     12,      3,     10,      6,     14,      8, &
    2,     13,      8,     11,      1 /)

  CASE(80)
  ellGrdHlp(1:145) = (/ &
   13,     17,     24,     19,     16,     16,     14,     19,      7,     18, &
   37,     17,     56,     30,     60,     -9,     80,    281,    973,    903, &
 1353,   1781,   1788,   1722,   1472,   1262,   1010,    839,    142,      8, &
   26,    103,    856,   1125,   1438,   1639,   1816,   1919,   1886,   1934, &
 1860,   1956,   2006,   1964,   1643,   1446,   1320,   1329,   1330,   1611, &
 1723,   1529,   1573,   1748,   1893,   1921,   1814,   1259,    674,     62, &
   51,    -47,    -72,    -44,    -50,    -52,    -51,    -61,    -53,    -53, &
  -48,    -61,    -63,    -61,    -66,    -49,    -63,    -55,    -66,    -49, &
  -61,    -54,    -49,    -52,    -51,    -57,    -53,    -68,    -42,    -58, &
  -49,    -47,    -35,    -44,    -30,    -37,    -26,    -38,    -33,    -44, &
  -22,    -35,    -27,    -25,    -14,    -31,     -5,    -24,     -6,    -18, &
   -7,    -26,    -12,     -4,    -24,     14,    -37,    604,    517,   -108, &
  123,    -45,     63,    -38,     30,     -7,     20,      4,     15,      5, &
    3,     10,      0,      9,     -1,     14,      4,     10,      9,      3, &
    5,      9,     14,     10,     13 /)

  CASE(81)
  ellGrdHlp(1:145) = (/ &
   16,     94,    144,    150,    138,     72,    208,    188,    109,    138, &
  164,    242,    462,    745,   1092,    905,   1289,   1853,   1998,   2242, &
 2411,   2650,   2804,   2727,   2608,   2347,   1909,   1048,    -37,    184, &
  979,   1638,   2161,   2382,   2599,   2682,   2674,   2732,   2770,   2688, &
 2539,   2475,   2453,   2478,   2490,   2435,   2389,   2366,   2439,   2541, &
 2552,   2422,   2359,   2402,   2465,   2502,   2384,   2138,   1831,   1655, &
 1660,   1577,   1362,   1244,    835,     55,    -16,    -86,    -85,    -48, &
  -46,    -63,    -62,    -80,    -56,    -73,    -46,    -72,    -60,    -54, &
  -68,    -59,    -54,    -48,    -57,    -51,    -62,    -70,    -46,    -55, &
  -55,    -39,    -39,    -44,    -35,    -32,    -23,    -21,    -41,    -24, &
  -22,    -31,    -30,    -14,    -13,     -5,     -7,     -0,    -23,      6, &
  -20,     -9,    -31,     19,     12,    261,    950,   1382,   2059,    962, &
   45,     21,     -8,      7,    -15,     26,    -28,     24,    -16,     16, &
   -8,      9,      1,      8,     -2,      5,      9,      1,      8,      7, &
   -6,     31,     11,     20,     16 /)

  CASE(82)
  ellGrdHlp(1:145) = (/ &
 1334,   2017,   2412,   2441,   2508,   2565,   2429,   2178,   1915,   1986, &
 2476,   2684,   2382,   2088,   2225,   2493,   2593,   2689,   2753,   2805, &
 2870,   2814,   2737,   2657,   2419,   2078,   1708,    943,    667,   1466, &
 2170,   2631,   2941,   3090,   3147,   3181,   3210,   3250,   3265,   3252, &
 3143,   2966,   2863,   2810,   2813,   2829,   2892,   2922,   2918,   2945, &
 2996,   3029,   2970,   2874,   2788,   2664,   2470,   2317,   2248,   2166, &
 2122,   2112,   2130,   2235,   2154,   1944,   1943,   1985,   1252,    -30, &
  -61,    -51,    -78,    -50,    -85,    -51,    -73,    -54,    -72,    -50, &
  -68,    -77,    -51,    -66,    -49,    -51,    -51,    -53,    -41,    -31, &
  -53,    -30,    -48,    -78,    -70,    -75,    -21,    -21,    -38,     -9, &
  -28,    -27,    -31,     69,    195,    151,     29,    -10,      3,     -0, &
   -4,    -15,     -2,      0,     54,    213,    357,    971,   2014,   1623, &
   94,    -13,    -11,    -15,     22,    -18,     10,    -13,     11,     -7, &
   10,     -6,     -4,      8,    -27,     12,     -9,      1,    -43,    192, &
  579,   1000,   1449,   1286,   1334 /)

  CASE(83)
  ellGrdHlp(1:145) = (/ &
 2708,   2744,   3256,   3720,   3843,   3721,   3527,   3574,   3646,   3359, &
 3257,   3269,   3164,   3126,   3078,   3068,   3065,   2945,   2895,   2869, &
 2790,   2669,   2508,   2370,   2177,   1773,   1428,   1494,   1961,   2500, &
 2903,   3041,   3069,   3242,   3344,   3431,   3500,   3468,   3509,   3483, &
 3348,   3223,   3098,   3056,   3064,   3057,   3080,   3081,   3038,   2986, &
 2955,   2919,   2846,   2783,   2747,   2698,   2629,   2539,   2439,   2325, &
 2223,   2191,   2203,   2129,   2023,   1814,   1187,    147,    -73,    -31, &
  -48,    -76,    -36,    -66,    -78,    -47,    -78,    -68,    -53,    -66, &
  -63,    -74,    -79,    -51,    -51,    -61,    -37,    -44,    -38,     -9, &
   12,    -23,    -89,     85,    150,     12,     48,     70,     81,     96, &
  -15,    -21,    -32,     80,    426,    686,    930,   1078,   1099,   1138, &
 1013,    805,    622,    501,    658,    990,   1357,   1687,   1567,    856, &
  136,    -36,    -21,     -8,      2,     -8,    -12,     -4,    -15,     -2, &
   10,    -14,      0,      8,     -9,     38,    322,   1145,   1561,   1812, &
 2105,   2422,   2853,   2982,   2708 /)

  CASE(84)
  ellGrdHlp(1:145) = (/ &
 2813,   2986,   3278,   3469,   3524,   3482,   3427,   3411,   3369,   3336, &
 3367,   3352,   3316,   3275,   3219,   3191,   3129,   3006,   2939,   2901, &
 2856,   2833,   2785,   2710,   2573,   2336,   2139,   2164,   2478,   2875, &
 3110,   3192,   3259,   3323,   3410,   3527,   3597,   3654,   3695,   3645, &
 3552,   3445,   3322,   3248,   3206,   3165,   3146,   3138,   3129,   3081, &
 3020,   2988,   2871,   2676,   2611,   2630,   2579,   2550,   2530,   2454, &
 2402,   2347,   2281,   2244,   1745,    839,    -16,    -49,    -21,   -111, &
  -53,    -15,   -100,    -66,    -31,    -73,    -68,    -56,    -66,    -41, &
  -53,   -102,    -83,    -62,    -30,     35,    278,    668,   1209,   1701, &
 1940,   1989,   2078,   2193,   2132,   1926,   1788,   1706,   1559,   1343, &
 1119,    967,    901,    872,    832,    826,    932,   1076,   1196,   1236, &
 1110,    929,    742,    506,    489,    816,    992,    639,    108,    -15, &
  -36,    -23,     -1,     -3,     -4,     16,      1,    -34,    -18,      6, &
  -15,    -27,     21,    228,    624,   1213,   1740,   2032,   2335,   2681, &
 2854,   2860,   2874,   2856,   2813 /)

  CASE(85)
  ellGrdHlp(1:145) = (/ &
 2891,   3073,   3262,   3346,   3375,   3391,   3383,   3364,   3369,   3388, &
 3387,   3363,   3353,   3359,   3367,   3382,   3393,   3391,   3344,   3208, &
 3036,   2928,   2880,   2847,   2862,   2933,   3011,   3085,   3172,   3250, &
 3301,   3344,   3392,   3472,   3593,   3689,   3708,   3685,   3671,   3659, &
 3619,   3545,   3463,   3382,   3272,   3165,   3125,   3122,   3085,   3048, &
 3043,   3008,   2913,   2801,   2685,   2587,   2565,   2551,   2452,   2368, &
 2368,   2288,   2177,   2262,   2217,   1580,    718,    226,     51,    -59, &
  -68,    -40,    -59,    -66,    -41,    -51,    -84,    -72,    -36,    -38, &
  -23,    168,    480,    669,    654,    632,    761,    966,   1144,   1320, &
 1528,   1704,   1787,   1800,   1791,   1774,   1749,   1701,   1621,   1542, &
 1510,   1532,   1551,   1535,   1577,   1707,   1727,   1649,   1794,   1995, &
 1557,    592,      9,    102,    163,    154,    125,     75,     11,    -19, &
  -28,    -48,     -4,    111,    213,    219,    154,    -19,    -79,     44, &
  286,    668,    886,   1081,   1392,   1746,   2049,   2366,   2665,   2789, &
 2788,   2841,   2896,   2867,   2891 /)

  CASE(86)
  ellGrdHlp(1:145) = (/ &
 2735,   2841,   3028,   3183,   3267,   3298,   3344,   3416,   3464,   3493, &
 3518,   3545,   3561,   3564,   3585,   3610,   3611,   3572,   3506,   3465, &
 3450,   3448,   3450,   3441,   3417,   3420,   3480,   3577,   3680,   3763, &
 3819,   3832,   3817,   3820,   3853,   3894,   3875,   3767,   3672,   3604, &
 3548,   3482,   3393,   3294,   3214,   3158,   3111,   3064,   3033,   3045, &
 2993,   2863,   2717,   2620,   2569,   2433,   2337,   2332,   2336,   2245, &
 2041,   2016,   2007,   1680,   1255,    205,   -218,    -88,      4,    -28, &
  -97,    -90,    -11,    -15,    -56,    -86,    -92,    -73,    -25,     27, &
   21,    -15,    -18,     43,    244,    388,    489,    563,    629,    712, &
  846,   1048,   1229,   1345,   1398,   1428,   1494,   1606,   1690,   1760, &
 1850,   1975,   2086,   2102,   2131,   2175,   2182,   2103,   1955,   1787, &
 1499,   1082,    634,    293,     32,    -25,    -24,     -9,      7,     29, &
   97,    192,    387,    658,    880,    889,    588,    154,     18,    -23, &
   45,    158,    474,    864,   1279,   1711,   2132,   2492,   2706,   2746, &
 2777,   2816,   2828,   2788,   2735 /)

  CASE(87)
  ellGrdHlp(1:145) = (/ &
 2718,   2829,   2941,   3057,   3116,   3165,   3205,   3251,   3299,   3356, &
 3424,   3491,   3547,   3582,   3620,   3665,   3723,   3758,   3778,   3775, &
 3776,   3781,   3799,   3836,   3888,   3967,   4029,   4074,   4071,   4053, &
 4016,   3971,   3927,   3909,   3903,   3880,   3804,   3666,   3524,   3414, &
 3373,   3361,   3346,   3284,   3204,   3119,   3082,   3054,   3034,   2954, &
 2869,   2781,   2727,   2665,   2571,   2456,   2364,   2314,   2246,   2163, &
 2093,   2121,   2140,   2046,   1624,   1261,    405,     63,    -76,    -87, &
  -47,    -18,     -3,    -29,    -55,    -84,    -75,    -67,    -56,    -64, &
  -54,     16,     77,    304,    388,    410,    400,    408,    470,    563, &
  644,    680,    698,    724,    838,   1013,   1233,   1429,   1598,   1735, &
 1909,   2085,   2251,   2292,   2276,   2194,   2143,   2089,   1985,   1772, &
 1469,   1148,    856,    649,    466,    374,    153,     85,     21,      3, &
   16,     32,    406,    569,    508,    495,    524,    649,    827,   1013, &
 1146,   1241,   1365,   1559,   1823,   2093,   2332,   2502,   2648,   2745, &
 2791,   2800,   2746,   2719,   2718 /)

  CASE(88)
  ellGrdHlp(1:145) = (/ &
 2708,   2746,   2784,   2848,   2911,   2962,   3013,   3056,   3099,   3157, &
 3218,   3284,   3351,   3406,   3459,   3496,   3532,   3559,   3587,   3618, &
 3647,   3673,   3699,   3717,   3733,   3748,   3762,   3757,   3750,   3698, &
 3646,   3581,   3517,   3486,   3454,   3441,   3427,   3398,   3368,   3322, &
 3277,   3239,   3202,   3183,   3164,   3134,   3103,   3045,   2987,   2945, &
 2903,   2862,   2821,   2749,   2678,   2659,   2641,   2669,   2698,   2640, &
 2579,   2522,   2465,   2549,   2643,   2658,   2673,   2378,   2078,   1653, &
 1226,   1155,    531,    329,    129,     20,    -82,   -100,   -131,   -113, &
  -91,    -31,     39,     71,    295,    369,    444,    467,    489,    547, &
  605,    696,    787,    854,    922,   1027,   1134,   1328,   1521,   1721, &
 1915,   2002,   2086,   2077,   2068,   2079,   2088,   2067,   2045,   1839, &
 1631,   1371,   1112,   1033,    963,    963,    965,    889,    807,    826, &
  847,   1052,   1258,   1384,   1509,   1485,   1462,   1503,   1543,   1653, &
 1762,   1830,   1897,   1991,   2089,   2255,   2421,   2558,   2691,   2731, &
 2770,   2750,   2730,   2719,   2708 /)

  CASE(89)
  ellGrdHlp(1:145) = (/ &
 2712,   2732,   2752,   2773,   2802,   2831,   2859,   2874,   2884,   2895, &
 2909,   2928,   2950,   2980,   3020,   3057,   3093,   3124,   3149,   3172, &
 3186,   3201,   3216,   3232,   3251,   3271,   3288,   3304,   3319,   3330, &
 3325,   3320,   3312,   3289,   3264,   3238,   3218,   3200,   3185,   3177, &
 3172,   3168,   3158,   3140,   3116,   3090,   3059,   3027,   2996,   2972, &
 2949,   2925,   2900,   2872,   2845,   2823,   2807,   2793,   2784,   2784, &
 2783,   2773,   2753,   2733,   2719,   2685,   2651,   2633,   2633,   2636, &
 2649,   2694,   2757,   2802,   2839,   2853,   2803,   2737,   2591,   2405, &
 2220,   2078,   1976,   1875,   1887,   1963,   2040,   2111,   2179,   2245, &
 2267,   2245,   2224,   2206,   2194,   2182,   2169,   2181,   2191,   2188, &
 2189,   2194,   2204,   2200,   2183,   2160,   2111,   2001,   1858,   1715, &
 1566,   1416,   1266,   1271,   1303,   1337,   1429,   1536,   1642,   1685, &
 1707,   1729,   1742,   1752,   1765,   1807,   1873,   1941,   2011,   2089, &
 2163,   2229,   2294,   2361,   2433,   2500,   2565,   2628,   2682,   2703, &
 2718,   2731,   2725,   2719,   2712 /)

  CASE(90)
  ellGrdHlp(1:145) = (/ &
 2774,   2791,   2808,   2826,   2843,   2860,   2877,   2892,   2907,   2923, &
 2938,   2952,   2967,   2977,   2986,   2995,   3003,   3010,   3018,   3022, &
 3024,   3027,   3029,   3031,   3033,   3033,   3035,   3036,   3037,   3038, &
 3038,   3039,   3038,   3039,   3039,   3039,   3039,   3034,   3028,   3022, &
 3016,   3011,   3005,   3002,   3001,   2999,   2998,   2996,   2994,   2997, &
 3000,   3003,   3005,   3007,   3010,   3005,   3000,   2994,   2989,   2984, &
 2979,   2985,   2992,   2998,   3005,   3012,   3020,   3040,   3060,   3080, &
 3100,   3120,   3140,   3134,   3129,   3123,   3117,   3112,   3106,   3073, &
 3035,   2996,   2957,   2919,   2880,   2839,   2798,   2756,   2716,   2675, &
 2634,   2621,   2609,   2596,   2584,   2575,   2566,   2572,   2578,   2584, &
 2590,   2596,   2603,   2590,   2576,   2563,   2550,   2536,   2523,   2519, &
 2516,   2513,   2510,   2506,   2503,   2517,   2530,   2544,   2557,   2570, &
 2584,   2583,   2581,   2579,   2577,   2575,   2574,   2576,   2577,   2578, &
 2581,   2584,   2587,   2601,   2615,   2630,   2644,   2659,   2675,   2692, &
 2708,   2725,   2741,   2758,   2774 /)

  CASE(91)
  ellGrdHlp(1:145) = (/ &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848,   2848, &
 2848,   2848,   2848,   2848,   2848 /)

  END SELECT
  ellGrd = ellGrdHlp(ii)
END FUNCTION ellGrd

END MODULE
